Package: GARCH.X
Title: Estimation and Exogenous Covariate Selection for ARCH-m(X),
        Additive ARCH-m(x), and GARCH-X Models
Version: 2.0
Description: Estimates the parameters and nonparametric functions of an ARCH-m(X) model with exogenous covariates, estimates the parameters and nonparametric functions of an Additive ARCH-m(X) model with exogenous covariates, estimates the parameters of a GARCH-X model with exogenous covariates, performs hypothesis tests for the covariates returning the p-values, and performs stepwise variable selection on the exogenous covariates, and uses False Discovery Rate p-value corrections to select the exogenous variables.
Authors@R: c(person(given = "Adriano",
                        family = "Zambom",
                        role = c("aut", "cre"),
                        email = "adriano.zambom@csun.edu"),
                 person(given = "Vincent",
                        family = "Alegrete",
                        role = "aut"),
                 person(given = "Elijah",
                        family = "Sagaran",
                        role = "aut"),
                 person(given = "Avni",
                        family = "Israni",
                        role = "aut"))
License: GPL (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, methods, nnls, utils, GA, GenSA, pso, KernSmooth
NeedsCompilation: no
Packaged: 2026-04-21 16:38:05 UTC; azambom
Author: Adriano Zambom [aut, cre],
  Vincent Alegrete [aut],
  Elijah Sagaran [aut],
  Avni Israni [aut]
Maintainer: Adriano Zambom <adriano.zambom@csun.edu>
Repository: CRAN
Date/Publication: 2026-04-21 17:12:08 UTC
Built: R 4.6.0; ; 2026-04-27 09:33:17 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(predict, ARCHmX)
S3method(predict, GARCHX)
S3method(predict, ARCHmXAdditive)
S3method(summary, ARCHmX)
S3method(summary, GARCHX)
S3method(summary, ARCHmXAdditive)
S3method(AIC, ARCHmX)
S3method(AIC, GARCHX)
S3method(AIC, ARCHmXAdditive)
S3method(BIC, ARCHmX)
S3method(BIC, GARCHX)
S3method(BIC, ARCHmXAdditive)
export(sim.ARCHmX)
export(sim.GARCHX)
export(sim.ARCHmXAdditive)
export(ARCHmX)
export(GARCHX)
export(ARCHmXAdditive)
export(stepwise)
importFrom("methods", "is")
importFrom("GA", "ga")
importFrom("GenSA", "GenSA")
importFrom("pso", "psoptim")
importFrom("nnls", "nnls")
importFrom("stats", "AIC", "BIC", "coef", "lm", "nlminb", "optim", "p.adjust", "pchisq", "pnorm", "predict", "rnorm", "rt", "runif", "sd", "var", "approx", "approxfun", "lowess")
importFrom("utils", "combn")
importFrom("KernSmooth", "locpoly")

summary.ARCHmXAdditive <- function(object, ...) {
  p <- object$p
  alphas <- object$alphas
  covariates <- object$covariates
  # p_values <- object$p_values
  sig_levels <- rep("", length(object$p.values))
  sig_levels[object$p.values <= 0.1] <- "."
  sig_levels[object$p.values <= 0.05] <- "*"
  sig_levels[object$p.values<= 0.01] <- "**"
  sig_levels[object$p.values <= 0.001] <- "***"
  
  cat("ARCHmXAdditive Model Summary:\n\n")
  #cat("Total training time:", object$train_time, "seconds.\n\n")
  
  cat("ARCH Coefficients:\n")
  alpha_names <- rep("alpha", p)
  for (i in 1:p){
    alpha_names[i] <- paste("alpha_", i, sep="")
  }
  alphas_df <- data.frame(
    names = alpha_names,
    alphas = alphas,
    pvalue = object$p.values[1:p],
    siglevel = sig_levels[1:p]
  )
  
  colnames(alphas_df)[ncol(alphas_df)] <- "" # when we have a column for p values
  
  print(alphas_df, row.names=FALSE)
  
  
  cat("\nResidual Deviation:", object$residual_deviation,"\n")
  
  if (length(covariates) > 0 && length(object$p.values) > p){ # WILL BE NULL FOR NOW for alpha p-values
    cat("\nExogenous Covariates Coefficients:\n")
    n_cov <- length(covariates)
    cov_names <- rep("", n_cov)
    for (i in 1:n_cov) {
      cov_names[i] <- paste("Covariate ", covariates[i], sep="")
    }
    covariate_df <- data.frame(
      names = cov_names,
      pvalue = object$p.values[(p + 1):length(object$p.values)],
      siglevel = sig_levels[(p + 1):length(sig_levels)]
    )
    colnames(covariate_df)[ncol(covariate_df)] <- ""
    print(covariate_df, row.names=FALSE)
  }
  else {
    cat("Covariate significance hypothesis test not run.\n")
  }
  cat("---\n")
  cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n")  
  
  
  cat("\n Information Criteria:\n")
  cat("AIC:", object$AIC,"\n")
  cat("BIC:", object$BIC,"\n")
  
}

GARCHX_check_order <- function(n, t, arr){
  # Used in GARCHX_variance function to get
  # h.entry or eps.entry

  # Check if n == 0
  if(n == 0){
    # If so, val is null
    val <- NULL
  } else {
    # If not, assign arr values to val
    val <- arr[(t-1):(t-n)]
  } # End if-else

  # Return val
  return(val)
} # End Check_p

# white noise simulator
ARCHmX_simulate_white_noise <- function(shock.distr) {
  if (shock.distr == "Normal") {
    obs <- rnorm(1, 0, 1)
  } else if (shock.distr == "t5") {
    obs <- rt(1, df = 5)
  } else if (shock.distr == "t7") {
    obs <- rt(1, df = 7)      
  } else { # Laplace distribution
    # Use fact that if X,Y ~ U(0,1), then log(X/Y) ~ Laplace(0,1)
    scale_param <- 0.5 # paper does scale parameter 0.5
    x <- runif(1,0,1)
    y <- runif(1,0,1)
    obs <- log(x/y) * scale_param
  }
  return(obs)
} # moved this outside of simulate and added scale param as per paper

ARCHmX <- function(eps, X, p, delta=2, covariates="all", hypothesisTest=TRUE, bandwidth = "plugin", k.choice = "plugin") {
  # Validate model parameters
  validate_ARCHmX(eps = eps, X = X, p = p,  delta = delta, covariates = covariates, bandwidth = bandwidth, k.choice = k.choice)
  
  # Call constructor to build the model
  obj <- new_ARCHmX(eps = eps, X = X, p = p, delta = delta, covariates = covariates, hypothesisTest = hypothesisTest, bandwidth = bandwidth, k.choice = k.choice)
  
  # Return the model
  return(obj)
}

predict.ARCHmXAdditive <- function(object, newdata, ...){
  # object is the fitted value returned by ARCHmXAdditive
  # newdata is the future covariate matrix (row = future time steps)
  
  # Validation
  if (is.null(object$covariates)) {
    stop("predict requires an ARCHmXAdditive object with a data component.")
  } else if (object$p == 0) {
    stop("predict requires p > 0.")
  } else if (is.null(object$m_funcs)) {
    stop("predict requires an ARCHmXAdditive object with m_funcs.")
  }
  
  # strip newdata to only the columns used in object
  newdata <- matrix(newdata[, object$covariates], nrow = nrow(newdata))
  
  # retrieve the necessary attributes from the model object
  eps <- object$eps # training residuals
  p <- object$p # order of ARCH
  alphas <- matrix(object$alphas, ncol = 1) # fitted ARCH coefficients
  T_ <- object$T_
  train <- matrix(object$X[, object$covariates], nrow = nrow(object$X))[(p+1):T_, , drop = FALSE]
  delta <- object$delta
  m_funcs <- object$m_funcs
  
  newdata <- rbind(matrix(train[nrow(train), ], nrow = 1), matrix(newdata[(1:(nrow(newdata)-1)), ], ncol = ncol(newdata)))
  
  m_hats <- rep(0, nrow(newdata))
  for (ell in 1:length(m_funcs)) {
    m_hats <- m_hats + m_funcs[[ell]](newdata[, ell])
  }
  # Ensure nonnegative
  m_hats <- pmax(m_hats, 0)
  
  # Expand eps^delta and generate the new predicted values
  eps_delta <- (abs(eps))^delta
  eps_delta <- c(eps_delta, rep(0, nrow(newdata)))
  
  # prediction
  for (i in 1:length(m_hats)) {
    eps_delta[T_ + i] <- eps_delta[(T_ + i - 1):(T_ + i - p)] %*% alphas + m_hats[i]
  }
  
  # return the new predicted volatilities
  return(eps_delta[(T_+1):(T_+length(m_hats))]^(2/delta))
}

sim.ARCHmX <- function(n, alpha, m, X, delta=2, shock.distr = "Normal", valinit=200) {
  # Validation
  # Note: We don't explicitly check whether m can be applied to X since
  #       we handle this in a tryCatch block later on in the function.
  
  if (!(shock.distr %in% c("Normal", "t5", "t7", "Laplace"))){
    stop("shock.distr must be Normal, t5, t7, or Laplace")
  } 
  if (length(alpha) == 0) {
    stop("alpha must contain at least a single parameter")
  }
  if (any(alpha < 0)) {
    stop("alpha must be nonnegative")
  }
  if (delta <= 0) {
    stop("delta must be a positive number.")
  }  
  if (!(is.matrix(X)) || (length(X) == 0)) {
    stop("X is not a valid data matrix.")
  }
  if (!is.function(m)){
    stop("m must be a function that accepts a matrix and returns a numeric vector.")
  }
  if(dim(X)[1] != (n +  valinit)){
    stop("Number of rows of X should be n + valinit")
  } 
  
  p <- length(alpha)
  epsilons <- rep(0, n + valinit)
  
  # Step 1: Initialization
  # ========================
  # We need an initial set of epsilons to actually 
  # build the iterative time series. 
  
  # for (i in 1:p) {
  #   epsilons[i] <- 0
  # }
  
  # Step 2: Burn-in period 
  # ========================
  # Here we first need to simulate the data matrix for 
  # our burn-in epsilons (we will actually create zero 
  # rows for the initial p-1 epsilons as well to keep  
  # the indices consistent later on)
  
  # covariate_means <- colMeans(X)
  # covariate_sds <- apply(X, 2, sd)
  # simulated_data <- matrix(0, nrow=p + burn_in_period, ncol=n_covariates)
  # for (i in p:(p + burn_in_period)) {
  #   for (j in 1:n_covariates) {
  #     simulated_data[i,j] <- rnorm(1, covariate_means[j], covariate_sds[j])
  #   }
  # }
  # 
  # # Then we will augment X with this simulated data
  # 
  # augmented_X <- rbind(simulated_data, X)
  
  # Next we compute the m function values:

  # tryCatch(
  #   {
  #     m_values <- m(X)
  #   }, error = function(msg){
  #     return(NA)
  #   })
  
  # assign m_values even on error so next validation block can cleanly exit if needed
  m_values <- tryCatch(
    m(X),
    error = function(e) NA
  )
  
  if (!is.numeric(m_values) || length(m_values) != n + valinit || any(!is.finite(m_values))) {
    stop("m(X) must return a finite numeric vector of length n + valinit.")
  }

  for (t in (p + 1):(n + valinit)) {
    sigma_to_deltath <- sum(alpha[1:p] * (epsilons^delta)[(t-1):(t-p)]) + m_values[t-1]
    if (!is.finite(sigma_to_deltath) || sigma_to_deltath <= 0) { 
      stop("non-positive or non-finite sigma^delta encountered during burn-in. check m and alpha.") 
    } # assumption in paper
    epsilons[t] <- ARCHmX_simulate_white_noise(shock.distr) * sigma_to_deltath^(1/delta)
  }
  
  # Step 3: Simulation
  # ========================  
  # Here we simulate the values to be returned
  
  # for (t in (p + burn_in_period + 1):(p + burn_in_period + n_samples)) {
  #   sigma_to_deltath <- sum(alpha[1:p] * (epsilons^delta)[(t-1):(t-p)]) + m_values[t-1]
  #   if (!is.finite(sigma_to_deltath) || sigma_to_deltath <= 0) { 
  #     stop("non-positive or non-finite sigma^delta encountered during burn-in. check m and alpha.") 
  #   } # assumption in paper
  #   epsilons[t] <- ARCHmX_simulate_white_noise(shock.distr) * sigma_to_deltath^(1/delta)
  # }  
  
  # And return them as a vector
  # return(epsilons[(p + burn_in_period + 1):(p + burn_in_period + n_samples)])
  i <- valinit + 1
  j <- valinit + n
  
  eps <- epsilons[i:j]
  X <- X[i:j, ]
  
  return(list(eps = eps, X = X))
}

validate_ARCHmXAdditive <- function(eps, X, p, delta, covariates, method, n_burn, k.choice) {
  if (!is.null(X)) 
  {
    # Error check: checks if X is a matrix
    if (!is.matrix(X)){
      stop("X must be a matrix with number of rows equal to the length of eps")
    } # End if
    if (nrow(X) != length(eps)) {
      stop("Data matrix X incompatible with epsilon vector.")
    }
    if (!identical(covariates,"all") & length(covariates) > ncol(X)) {
      stop("Data matrix X incompatible with covariate list.")
    }
  }
  if ((k.choice != "plugin") && (k.choice != "CV")) {
    stop("bandwidth should be CV or plugin.")
  }
  if (!is.numeric(p) || length(p) != 1 || p <= 0) {
    stop("p must be a positive integer.")
  }
  if (delta <= 0) {
    stop("delta must be a positive number.")
  }  
  if (length(eps) <= p + n_burn){
    stop("Need length(eps) > p + n_burn.")
  } 
  if (!is.character(method) || length(method) != 1 || !(method %in% c("smooth_means_Positive_Constraint_Exp", "smooth_means_lowes", "smooth_variance"))) {
    stop("method must be one of 'smooth_means_Positive_Constraint_Exp', 'smooth_means_lowes', 'smooth_variance'.")
  }
  
}

ARCHmXAdditive <- function(eps, X, p, delta=2, covariates="all", 
                   method = "smooth_means_Positive_Constraint_Exp", ## one of  "smooth_means_Positive_Constraint_Exp", "smooth_means_lowes", "smooth_variance"
                   train_end = NULL, n_burn = 10, max_outer_iter = 30, tol = 1e-4,
                   hypothesisTest = TRUE, k.choice = "plugin"
                   ) { # last 2 params are for variable selection
  
  # questions
  # should i get rid of n_burn and just do same default amount no matter what? -- new_ARCHmX doesnt hv this

  # Validation
  validate_ARCHmXAdditive(eps = eps, X = X, p = p, delta = delta, covariates = covariates, method = method, n_burn = n_burn,  k.choice = k.choice)
  
  # Call constructor to build the model
  obj <- new_ARCHmXAdditive(eps = eps, X = X, p = p, delta = delta, covariates = covariates,
    method = method, train_end = train_end, n_burn = n_burn, max_outer_iter = max_outer_iter, 
    tol = tol, hypothesisTest = hypothesisTest, k.choice = k.choice
  )
  
  # Return the model
  return(obj)
}

# Function to choose k according to Steps 1, 2, and 3 in the paper
ARCHmX_choosek = function(X, Y, range, B, alpha)
{
  rate=rep(0,length(range))
  for(l in 1:length(range))
  {
    reject = 0
    for(i in 1:B)
    {
      newresi=sample(Y,length(Y),replace=FALSE)
      if (ARCHmX_anova_test_univariate(X, abs(newresi),range[l]) < 0.05)
        reject = reject+1
    }
    rate[l]=abs(sum(reject/B)-alpha)
  }
  range[rate==min(rate)][1] #Choose smaller window size if more than one window provides the same performance.
  
}

# --- 10-fold CV to choose best h from grid --------------------------------
# X: n x d (rows = obs, cols = covariates)
# y: length n
# h_grid: numeric vector of candidate bandwidths
# nfolds: default 10
# seed: optional for reproducible folds
ARCHmX_nw_cv_bandwidth <- function(X, y, h_grid, nfolds = 10, seed = NULL, shuffle = TRUE) {
  stopifnot(is.matrix(X), length(y) == nrow(X), length(h_grid) >= 1)
  
  n <- nrow(X)
  idx <- seq_len(n)
  if (shuffle) {
    if (!is.null(seed)) set.seed(seed)
    idx <- sample.int(n)
  }
  # make folds as contiguous chunks of idx
  folds <- split(idx, rep_len(seq_len(nfolds), n))
  
  # CV error for each h (MSE, weighted by fold sizes)
  cv_mse <- numeric(length(h_grid))
  
  for (k in seq_along(h_grid)) {
    h <- h_grid[k]
    se_sum <- 0
    n_sum  <- 0
    
    for (f in seq_along(folds)) {
      val_idx <- folds[[f]]
      train_idx <- setdiff(seq_len(n), val_idx)
      
      Xtr <- X[train_idx, , drop = FALSE]
      ytr <- y[train_idx]
      Xva <- X[val_idx, , drop = FALSE]
      yva <- y[val_idx]
      
      yhat <- ARCHmX_predict_nw_iso(Xtr, ytr, Xva, h)
      se_sum <- se_sum + sum((yva - yhat)^2)
      n_sum  <- n_sum  + length(val_idx)
    }
    
    cv_mse[k] <- se_sum / n_sum
  }
  
  best_idx <- which.min(cv_mse)
  list(
    best_h = h_grid[best_idx],
    cv_mse = cv_mse,
    h_grid = h_grid,
    predict = function(Xnew, Xtrain = X, ytrain = y) {
      # convenience predictor using the chosen h
      ARCHmX_predict_nw_iso(Xtrain, ytrain, Xnew, h_grid[best_idx])
    }
  )
}

validate_GARCHX <- function(eps, X, order){
  # Fit the model and returns all the p-values
  
  # Just one pass of estimate
  
  # Declaring initial values for paramters
  # d = number of exogenous covariates
  if (!is.null(X))
  {
    # Error check: checks if X is a matrix
    if (!is.matrix(X)){
      stop("X must be a matrix with number of rows equal to the length of eps")
    } # End if
    
    # Error check: checks if number of rows of X is equal to length of eps
    if(dim(X)[1] != (length(eps))){
      stop("Number of rows of matrix X must be the same as eps")
    } # End if
  }
  
  # Error check: checks of order is a vector of length 2 for p and q values
  if (!is.vector(order) || length(order) != 2){
    stop("order must be a vector of length 2 with entries determining the order of the GARCH model")
  } # End if
  
  # Error check: order is c(0, 0)
  if(order[2] < 0){
    stop("q must be a non negative integer")
  } # End if
  
  if(order[1] <= 0){
    stop("p must be greater than 0")
  } # End if
} 

GARCHX_objf <- function(coef, delta, order, d, r, sigma.init, eps, n, X, eps.delta){
  # The same function as Likelihood
  # Assign omega, alpha, beta, and gamma coefficients values
  # Same question, should we do the same thing we did for GARCHX.select.default
  p <- order[1]
  q <- order[2]
  omega <- coef[1]
  alpha <- coef[2:(p+1)]
  
  if(q != 0){
    beta <- coef[(p+2):(p+q+1)]
  } else {
    beta <- 0
  }
  if(d != 0){
    gamma <- coef[(p+q+2):(1+p+q+d)]
  } else {
    gamma <- 0
  }
  # Initialize h
  h <- rep(0, n)
  h[1:(max(p,q))] <- sigma.init^{delta/2}


  for(t in (2+(max(p,q))-1):n)
  {
    if (d == 0)
      #h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + beta %*% h[(t-1):(t-q)]
      h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + sum(beta %*% h[(t-1):(t-q)])
    else
      #h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + beta %*% h[(t-1):(t-q)] +  drop(X[t-1,] %*% gamma)
      h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + sum(beta %*% h[(t-1):(t-q)]) +  drop(X[t-1,] %*% gamma)
    
  } # End for
  
  sigma <- h^(2/delta)

  qml <- mean(log(sigma[(r+1):n]) + eps[(r+1):n]^2/sigma[(r+1):n])

  qml
} # End objf

GARCHX <- function(eps, X, order = c(1, 1), delta = 2, covariates = "all", optim.method = "NR"){
  validate_GARCHX(eps, X, order)
  obj <- new_GARCHX(eps, X, order, delta, covariates, optim.method)
  return(obj)
}

ARCHmX_anova_test_univariate <- function(X, Y, k_n)   # include the edges
{
  # VCA: X is the covariate we are testing, contains the x_t
  # VCA: Y is the vector of residuals v_t corresponding to each x_{t-1}
  n = length(X)
  xi = Y
  # VCA: the residuals v_t must be ordered in terms of increasing x_{t-1} per Paper_EJS pg. 4
  # 10/21 - when called from choosek, the random permutation of v_t means that the
  # order is still randomized even after this sort.
  xi = Y
  xi[!is.finite(xi)] = 0 # added this 4/2 - [AI]
  xi = xi[order(X)]
  
  # VCA: Each k_n-long window is a row in matrix V
  V = matrix(0, n-k_n+1, k_n)
  for(ind in ((k_n+1)/2):(n-(k_n+1)/2+1))                # AZZ: I am excluding the edges
    V[ind-(k_n+1)/2+1,] = xi[(ind-(k_n+1)/2+1):(ind+(k_n+1)/2-1)]
  
  nv = dim(V)[1] # VCA: This is the number of rows / windows
  t_mean = mean(V) # VCA: This is the overall mean v..
  # 10/19 checked this matches the definition under (2.6) on Paper_EJS pg 5
  
  MST = sum((rowMeans(V) - t_mean)^2)*k_n/(nv-1)
  MSE = sum((V - rowMeans(V))^2)/(nv*k_n - nv)
  
  #   tau = sum((xi[2:(n-2)] - xi[1:(n-3)])^2*(xi[4:n] - xi[3:(n-1)])^2)/(4*(n-3))
  tau = sum((xi[2:n] - xi[1:(n-1)])^2)/(2*(n-1))
  # added is.na(tau) check - [AI]
  if (is.na(tau) || tau == 0) ### when the estimate interpolates, tau = 0, and an error would occur, so we set it > 0
    tau = 0.0000000000001
  
  
  #   p_value = (1-pnorm((sqrt(nv)*(MST-MSE))/(sqrt(2*k_n*(2*k_n-1)*tau/(3*(k_n-1))))))
  p_value = (1-pnorm((sqrt(nv/k_n)*(MST-MSE))/(sqrt(4*tau^2/3))))
  
  return(p_value)
}

# approach C: stabilize residuals, compute local weighted variance, then take square root and shift to enforce nonnegative

# --- Weighted variance with a small-sample correction (Bessel-like) ---
wvar <- function(y, w) {
  w <- pmax(w, 0) # ensure weights nonnegative
  sw  <- sum(w)
  if (sw <= .Machine$double.eps) return(NA_real_)
  m   <- sum(w * y) / sw # weighted mean
  num <- sum(w * (y - m)^2) # weighted sum of squares around the mean
  # Unbiased-ish denom: sum(w) - sum(w^2)/sum(w)
  denom <- sw - sum(w^2) / sw
  if (denom > .Machine$double.eps) num / denom else num / sw
}

# --- kernels (univariate) ---
kern_fun <- function(u, kernel = c("gaussian","epanechnikov")) {
  kernel <- match.arg(kernel)
  if (kernel == "gaussian") {
    exp(-0.5*u^2) / sqrt(2*pi)
  } else {
    out <- 0.75 * (1 - u^2)
    out[abs(u) > 1] <- 0
    out
  }
}

# --- Moving-window variance with kernel weighting (fixed bandwidth h) ---
moving_var_kernel <- function(x, y, x0 = x, h, kernel = c("gaussian","epanechnikov"),
                              min_pts = 5) {
  kernel <- match.arg(kernel)
  x  <- as.numeric(x); y <- as.numeric(y); x0 <- as.numeric(x0)
  stopifnot(length(x) == length(y))
  if (length(h) != 1 || h <= 0) stop("Provide a positive scalar bandwidth h.")
  out <- numeric(length(x0))
  for (i in seq_along(x0)) {
    u  <- (x - x0[i]) / h # kernel scaling
    w  <- kern_fun(u, kernel = kernel)
    # ensure enough effective points
    if (sum(w > 0) < min_pts) {
      out[i] <- NA_real_
    } else {
      out[i] <- wvar(y, w)
    }
  }
  out
}

ARCHmXAdditive_backfitting_variance_smooth <- function(target_sum_nonpar, X, bandwidth = NULL,
                                        max_iter = 50, tol = 1e-4, floor_eps = 1e-8) {
  # target_sum_nonpar: vector of length n with the "total" nonparametric component
  #                    i.e., sigma2_resid = eps^2 - arch_component   (can be < 0)

  n <- nrow(X) # number of samples
  d <- ncol(X) # number of covariates
  
  # Initialize m values strictly positive (tiny), and storage for functions
  m_values <- matrix(floor_eps, nrow = n, ncol = d)
  m_funcs  <- vector("list", d)
  
  
  for (iter in 1:max_iter) {
    m_old <- m_values
    
    for (ell in 1:d) {
      # Partial residual for component ell
      partial_resid <- target_sum_nonpar - rowSums(m_values[, -ell, drop = FALSE])
      
      ########################################    
      # Variance Smoothing
      
      x = X[,ell]
      xx <- x #seq(min(x), max(x), length.out = 300)
      
      # Bandwidth via Silverman heuristic (works fine for illustration)
      h  <- 1.06 * sd(x) * n^(-1/5)
      
      # Kernel moving-variance (Gaussian kernel)
      v_hat_kernel <- moving_var_kernel(x, partial_resid, x0 = xx, h = h, kernel = "gaussian", min_pts = 10)
      #v_hat_kernel <- moving_var_kernel(x, sqrt(partial_resid - min(partial_resid)), x0 = xx, h = h, kernel = "gaussian", min_pts = 10)
      
      y_hat = sqrt(v_hat_kernel)-min(sqrt(v_hat_kernel))
      
      #cat("\n ####### backfitting var smooth ###### ell = ", ell)
      #cat("\n x = ",x[1:10])
      #cat("\n target = ", target_sum_nonpar[1:10])
      #cat("\n y_hat = ",y_hat[1:10])
      
      fit = list()
      fit$x = xx
      fit$y = y_hat
      ########################################        
      
      
      
      # Predict on observed X and exponentiate back
      ghat_at_x  <- approx(fit$x, fit$y, xout = X[, ell], rule = 2)$y
      m_values[, ell] <- ghat_at_x
      
      # Store the function on a grid
      m_funcs[[ell]] <- (function(xgrid, ygrid) {
        # returns m_ell(x) = exp(g_ell(x))
        f_lin <- approxfun(xgrid, ygrid, rule = 2)
        function(z) f_lin(z)
      })(fit$x, fit$y)
    }
    
    if (max(abs(m_values - m_old)) < tol) break
  }
  
  list(m_values = m_values, m_funcs = m_funcs, bandwidth = bandwidth)
}

BIC.GARCHX <- function(object, ...){
  # Calculate initial value of sigma^2
  #sigma2.init <- mean(object$eps[1:object$r]^2)
  sigma.init <- mean(object$eps[1:object$r]^object$delta)

  # Calculate epsilon^delta
  eps.delta <- object$eps^object$delta

  # Check if the object is a GARCHX or GARCH object
  if(is.na(object$gamma[1])){
    null = FALSE
  } else {
    null = TRUE
  } # End if-else

  # Calculate QMLE of GARCHX object
  # QMLE <- Likelihood(object, sigma.init = sigma.init, length(object$eps),
  #                    eps.delta = eps.delta, null = null)
  # Should nlminb be used here?

  p <- object$order[1]
  q <- object$order[2]
  # Calculate BIC of object
  BIC <- 2 * length(object$eps) * object$Likelihood + (1 + p + q + object$d) * log(length(object$eps))

  return(BIC)
} # End BIC.GARCHX

new_ARCHmXAdditive <- function(eps, X, p, delta=2, covariates="all", 
                           method = "smooth_means_Positive_Constraint_Exp", ## one of  "smooth_means_Positive_Constraint_Exp", "smooth_means_lowes", "smooth_variance"
                           train_end = NULL, n_burn = 10, max_outer_iter = 30, tol = 1e-4,
                           hypothesisTest = TRUE, k.choice = "plugin")
{
  
  
  

  
  
  # Typing
  #X <- as.matrix(X)
  eps <- as.numeric(eps)
  p <- as.integer(p)
  N <- length(eps)
  
  if ((!identical(covariates,"all")) || (is.null(X))) 
  {
    if ((((covariates[1] == 0) || is.null(covariates))) || (is.null(X))) 
    {
      return(GARCHX(eps=eps, X = NULL, order = c(p,0), delta=delta))
    } else 
    {
      if (max(covariates) > ncol(X))
      {
        stop("Data matrix X incompatible with covariate list.")
      } 
      X <- X[, covariates, drop = FALSE]
    }
  } else { #covariates = "all" so change to a vector
    covariates <- 1:ncol(X)
  }
  
  if (!is.null(train_end)) {
    if (!is.numeric(train_end) || length(train_end) != 1){
      stop("train_end must be a single integer or NULL.")
    } 
    train_end <- as.integer(train_end)
    if (train_end <= p + n_burn){
      stop("train_end must be > p + n_burn.")
    } 
    if (train_end > N){
      stop("train_end must be <= length(eps).") 
    }
  }
  
  #method <- match.arg(method)
  #backfit.method <- switch(
  #  method,
  #  "A" = "smooth_means_Positive_Constraint_Exp",
  #  "B" = "smooth_means_lowes",
  #  "C" = "smooth_variance"
  #)
  
  # Fit model
  
  fit <- ARCHmXAdditive_estimate(
    eps = eps,
    X = X,
    p = p,
    delta = delta,
    train_end = train_end,
    n_burn = n_burn,
    max_outer_iter = max_outer_iter,
    tol = tol,
    backfit.method = method
  )
  

  # AIC, BIC
  AIC = -2 * fit$loglik + 2*p # TO DO: will find correct df
  BIC = -2 * fit$loglik + p*log(length(eps)) # TO DO: will find correct df 
  
  v_hat = eps^2 - fit$sigma_hat^2
  v_hat[!is.finite(v_hat) | is.na(v_hat)] = 0 
  residual_deviation <- sqrt(mean(v_hat^2)) 
  
  #################################################
  # Hypothesis testing for variable significance  
  #################################################
  
  alpha_p.values <- rep(NA_real_, p) # TBD!!
  covariate_p.values <- NULL
  p.values <- NULL
  
  if (hypothesisTest) {
    
    n_vars <- ncol(X)
    v_hats <- c()
    for (i in 1:n_vars) 
    {
      X_prime <- matrix(X[, -i], nrow=nrow(X))
      if (length(X_prime) > 0) {
        
        fit_prime <- new_ARCHmXAdditive(X = X_prime, p = p, eps = eps, delta = delta, covariates = "all",
                                    method = method, 
                                    train_end = train_end, n_burn = n_burn, max_outer_iter = max_outer_iter, tol = tol,
                                    hypothesisTest = FALSE, k.choice = k.choice
        ) # hypothesisTest = FALSE to prevent endless recursive loop
        
        v_hat_prime <- eps^2 - fit_prime$sigma_hat^2
        v_hat_prime[!is.finite(v_hat_prime) | is.na(v_hat_prime)] = 0 # must ensure not NA otherwise ARCHmX_anova_test_univariate won't work since Y might have nulls
        v_hats <- c(v_hats, v_hat_prime)
      }
      else {  # no covariates (we can just use ARCH_internal then)
        empty_model <- GARCHX(eps = eps, X=NULL, order = c(p,0), delta = delta)   # ARCH_internal(eps, p, delta = delta)
        v_hat_prime = empty_model$v_hat
        v_hat_prime[!is.finite(v_hat_prime) | is.na(v_hat_prime)] = 0 # must ensure not NA otherwise ARCHmX_anova_test_univariate won't work since Y might have nulls
        v_hats <- c(v_hats, v_hat_prime)
      }
    }
    
    # Now, let's perform the tests
    B <- 200
    alpha <- 0.05
    covariate_p.values <- rep(0, n_vars)
    
    next_odd <- function(n) {
      n <- floor(n)                 # floor each element
      ifelse(n %% 2 == 0, n + 1, n) # if even → next odd; if odd → keep
    }
    range <- unique(next_odd(seq(next_odd(N^(1/3)/2), next_odd(N^(1/3)), length = 5)))
    
    for (i in 1:n_vars){ # use the ARCHmX methods
      X_values <- X[,i][1:(N - 1)]
      offset <- N * (i-1)
      Y_values <- v_hats[(offset + 2):(offset + N)]
      if (k.choice == "plugin")
        k_n = next_odd(N^(29/90))
      else
        k_n <- ARCHmX_choosek(X_values, Y_values, range = range, B = B, alpha = alpha)
      
      covariate_p.values[i] <- ARCHmX_anova_test_univariate(X_values, Y_values, k_n) # relationship between removed variable and v_hat
    }
  }
  
  if (is.null(covariate_p.values)) {
    covariate_p.values <- numeric(0)
  }
  
  p.values <- c(alpha_p.values, covariate_p.values)
  

  #################################################
  # Set model parameters 
  #################################################
  
  structure(
    list(
      N = N,
      covariates = covariates, # vector of numbers indexing the covariates to be used
      method = method, # ## one of  "smooth_means_Positive_Constraint_Exp", "smooth_means_lowes", "smooth_variance"
      p = p, # order of ARCH
      X = X, # matrix of covariates
      eps = eps, 
      delta=delta,
      
      alphas = fit$alphas,
      m_funcs = fit$m_funcs, # list of callable functions
      m_hats = fit$m_hats,
      
      AIC = AIC, 
      BIC = BIC, 
      v_hat = v_hat,
      residual_deviation = residual_deviation, 
      
      sigma_hat = fit$sigma_hat,
      
      n_burn = fit$n_burn,
      
      log_QL = fit$loglik,
      converged = fit$converged,
      

      hypothesisTest = hypothesisTest, # variable-selection related 
      k.choice = k.choice,
      p.values = c(p.values), #this will become p_values for alphas not covariates
      covariate_p.values = c(covariate_p.values),
      
      train_end = train_end, # added 2/6/26
      max_outer_iter = max_outer_iter, 
      tol = tol
      ),
    class = "ARCHmXAdditive"
  )
}

# helper to compute nonparametric additive term 
m_sum <- function(row_vector, m, d) {
  s <- 0
  for (ell in 1:d) {
    val <- tryCatch(m[[ell]](row_vector[ell]), error = function(e) NA)
    if (!is.numeric(val) || length(val) != 1 || is.na(val) || is.infinite(val)) {
      stop(paste0("m[[", ell, "]] must return a single finite numeric value"))
    }
    if (val < 0){
      stop(paste0("m[[", ell, "]] must be nonnegative.")) # assumption in paper
    } 
    s <- s + val
  }
  return(s)
}

sim.ARCHmXAdditive <- function(n, alpha, m, X,  delta=2, valinit = 200, shock.distr = "Normal") {
  # Validation
  
  # NOTE: It is assumed that each m produces positive output. If non-positive, the function exits.
  # This occurs in a later check. 
  
  if (!(shock.distr %in% c("Normal", "t5", "t7", "Laplace"))){
    stop("shock.distr must be Normal, t5, t7, or Laplace")
  } 
  if (length(alpha) == 0) {
    stop("alpha must contain at least a single parameter")
  }
  if (any(alpha < 0)) {
    stop("alpha must be nonnegative") # assumption in paper 
  } 
  if (delta <= 0) {
    stop("delta must be a positive number.")
  }  
  if (!(is.matrix(X)) || (length(X) == 0)) {
    stop("X is not a valid data matrix.")
  }
  if(!is.list(m) || length(m) != dim(X)[2]){
    stop("m must be a list with the same number of covariates as in X.")
  }
  if (any(!vapply(m, is.function, logical(1)))) {
    stop("all elements of m must be functions.")
  }
  if(dim(X)[1] != (n +  valinit)){
    stop("Number of rows of X should be n + valinit")
  }
  
  p <- length(alpha) # number of ARCH lags
  d <- dim(X)[2] # number of covariates
  epsilons <- rep(0, n + valinit)
  
  # Step 2: Burn-in period 
  # ========================
  # Here we first need to simulate the data matrix for 
  # our burn-in epsilons (we will actually create zero 
  # rows for the initial p-1 epsilons as well to keep  
  # the indices consistent later on)
  
  # covariate_means <- colMeans(X)
  # covariate_sds <- apply(X, 2, sd)
  # simulated_data <- matrix(0, nrow=p + valinit, ncol=d)
  # for (i in p:(p + valinit)) {
  #   for (j in 1:d) {
  #     simulated_data[i,j] <- rnorm(1, covariate_means[j], covariate_sds[j])
  #   }
  # }
  # 
  # # Then we will augment X with this simulated data
  # 
  # augmented_X <- rbind(simulated_data, X)
  
  # Next we perform the burn-in simulations: 
  for (t in (p + 1):(n + valinit)) {
    sigma_to_deltath <- sum(alpha[1:p] * (epsilons^delta)[(t-1):(t-p)]) + m_sum(X[t-1, ], m, d)
    if (!is.finite(sigma_to_deltath) || sigma_to_deltath <= 0) { 
      stop("non-positive or non-finite sigma^delta encountered during burn-in. check m and alpha.") 
    } # assumption in paper
    epsilons[t] <- ARCHmX_simulate_white_noise(shock.distr) * sigma_to_deltath^(1/delta)
  }
  
  # Step 3: Simulation
  # ========================  
  # Here we simulate the values to be returned
  
  # for (t in (p + valinit + 1):(p + valinit + n_samples)) {
  #   sigma_to_deltath <- sum(alpha[1:p] * (epsilons^delta)[(t-1):(t-p)]) + m_sum(augmented_X[t-1, ], m, d)
  #   if (!is.finite(sigma_to_deltath) || sigma_to_deltath <= 0) { 
  #     stop("non-positive or non-finite sigma^delta encountered during burn-in. check m and alpha.") 
  #   } # assumption in paper
  #   epsilons[t] <- ARCHmX_simulate_white_noise(shock.distr) * sigma_to_deltath^(1/delta)
  # }  
  
  # And return them as a list
  i <- valinit + 1
  j <- valinit + n
  
  eps <- epsilons[i:j]
  X <- X[i:j, ]
  
  return(list(eps = eps, X = X))
}

qmle_loglik <- function(params, eps, X, p, delta, m_funcs, start_idx = 1) {
  n <- length(eps)
  d <- ncol(X)
  
  
  alphas <- params[1:(p)]
  
  if ( any(alphas < 0) || sum(alphas) >= 1) {
    return(-1e10)
  }
  
  sigma2 <- numeric(n)
  sigma2[1:p] <- var(eps[start_idx:n])
  
  for (t in max(p+1, start_idx):n) {
    arch_component <-  sum(alphas * eps[(t-1):(t-p)]^2)
    nonpar_component <- 0
    for (ell in 1:d) {
      x_val <- X[t-1, ell]
      nonpar_component <- nonpar_component + m_funcs[[ell]](x_val)
    }
    sigma2[t] <- arch_component + nonpar_component
    if (sigma2[t] <= 0) sigma2[t] <- 1e-6
  }
  
  valid_idx <- max(p+1, start_idx):n
  loglik <- -0.5 * sum(log(2 * pi * sigma2[valid_idx]) + 
                         eps[valid_idx]^2 / sigma2[valid_idx])
  
  return(loglik)
}


######################
ARCHmXAdditive_estimate <- function(eps, X, p, delta, train_end = NULL, n_burn = 10, 
                                   max_outer_iter = 30, tol = 1e-4, 
                                   backfit.method = "smooth_means_Positive_Constraint_Exp") 
{
  n <- length(eps)
  d <- ncol(X)
  if (is.null(train_end)) 
    train_end <- n
  start_idx <- n_burn + 1
  
  # init m's as zero functions (will become positive)
  m_funcs <- vector("list", d)
  for (ell in 1:d) m_funcs[[ell]] <- function(x) 0
  
  
  alphas_init <- rep(0.05, p) # rep(0.6 / p, p)
  params_init <- c( alphas_init)
  converged_outer <- FALSE
  
  eps_train <- eps[1:train_end]
  X_train <- X[1:train_end, , drop = FALSE]
  
  for (outer_iter in 1:max_outer_iter) {
    params_old <- params_init
    
    optim_result <- optim(
      par = params_init,
      fn = function(params) -qmle_loglik(params, eps_train, X_train, p, delta, m_funcs, start_idx),
      method = "L-BFGS-B",
      lower = c(1e-6, rep(0, p)),
      upper = c(Inf, rep(0.99, p)),
      control = list(maxit = 2000, factr = 1e7)
    )
    
    params_new <- optim_result$par
    alphas <- params_new[1:(p)]
    
    # ----------- CHANGED: build target sum for nonparam part (can be negative) -----------
    sigma2_resid <- numeric(train_end)
    sigma2_resid[1:p] <- NA
    for (t in (p+1):train_end) {
      arch_component <- sum(alphas * eps_train[(t-1):(t-p)]^2)
      sigma2_resid[t] <- eps_train[t]^2 - arch_component
    }
    # Valid training slice and lagged X for m_ell(X_{t-1,ell})
    valid_idx <- start_idx:train_end
    X_lagged <- X_train[valid_idx - 1, , drop = FALSE]
    resid_valid <- sigma2_resid[valid_idx]
    
    # ----------- CHANGED: positive backfitting on log scale -----------------------------
    if (backfit.method == "smooth_means_Positive_Constraint_Exp")
      backfit_result <- ARCHmXAdditive_backfitting(resid_valid, X_lagged, max_iter = 100)
    else if (backfit.method == "smooth_means_lowes")
      backfit_result <- ARCHmXAdditive_backfitting_lowes(resid_valid, X_lagged, max_iter = 100)
    else if (backfit.method == "smooth_variance")
      backfit_result <- ARCHmXAdditive_backfitting_variance_smooth(resid_valid, X_lagged, max_iter = 100)
    m_funcs <- backfit_result$m_funcs
    
    # Check parameter change
    if (max(abs(params_new - params_old)) < tol) {
      converged_outer <- TRUE
      break
    }
    params_init <- params_new
  }
  
  optim_result <- optim(
    par = params_init,
    fn = function(params) -qmle_loglik(params, eps_train, X_train, p, delta, m_funcs, start_idx),
    method = "L-BFGS-B",
    lower = c(1e-6, rep(0, p)),
    upper = c(Inf, rep(0.99, p)),
    control = list(maxit = 2000, factr = 1e7)
  )
  params_final <- optim_result$par
  alphas <- params_final[1:(p)]
  
  # Fitted sigma2 over ALL data
  sigma2_hat <- numeric(n)
  sigma2_hat[1:p] <- NA
  for (t in (p+1):n) {
    arch_component <- sum(alphas * eps[(t-1):(t-p)]^2)
    nonpar_component <- 0
    for (ell in 1:d) nonpar_component <- nonpar_component + m_funcs[[ell]](X[t-1, ell])
    sigma2_hat[t] <- max(arch_component + nonpar_component, 1e-6)
  }
  
  sigma_hat <- sqrt(sigma2_hat)
  sigma_realized <- abs(eps)
  
  # Metrics (unchanged) ...
  
  # Export m-hats on a grid
  m_hats <- list()
  for (ell in 1:d) {
    x_grid <- seq(min(X[, ell]), max(X[, ell]), length.out = 100)
    m_grid <- sapply(x_grid, m_funcs[[ell]])
    m_hats[[ell]] <- list(x = x_grid, m = m_grid)
  }
  
  list(
    alphas = alphas,
    m_hats = m_hats,
    m_funcs = m_funcs,
    sigma_hat = sigma_hat,
    #sigma_realized = sigma_realized,
    #MAE_train = mean(abs(sigma_hat[(start_idx):train_end] - sigma_realized[(start_idx):train_end]), na.rm=TRUE),
    #MAPE_train = mean(abs((sigma_hat[(start_idx):train_end] - sigma_realized[(start_idx):train_end]) / 
    #                        sigma_realized[(start_idx):train_end]), na.rm=TRUE) * 100,
    #RMSE_train = sqrt(mean((sigma_hat[(start_idx):train_end] - sigma_realized[(start_idx):train_end])^2, na.rm=TRUE)),
    #MAE_test = if (train_end < n) mean(abs(sigma_hat[(train_end+1):n] - sigma_realized[(train_end+1):n]), na.rm=TRUE) else NA,
    #MAPE_test = if (train_end < n) mean(abs((sigma_hat[(train_end+1):n] - sigma_realized[(train_end+1):n]) / 
    #                                          sigma_realized[(train_end+1):n]), na.rm=TRUE) * 100 else NA,
    #RMSE_test = if (train_end < n) sqrt(mean((sigma_hat[(train_end+1):n] - sigma_realized[(train_end+1):n])^2, na.rm=TRUE)) else NA,
    loglik = -optim_result$value,
    converged = converged_outer,
    #train_end = train_end,
    n_burn = n_burn
  )
}

summary.GARCHX <- function(object, ...) {
  p <- object$order[1]
  q <- object$order[2]
  covariates <- object$covariates
  sig_levels <- rep("", length(object$p.values))
  sig_levels[object$p.values <= 0.1] <- "."
  sig_levels[object$p.values <= 0.05] <- "*"
  sig_levels[object$p.values<= 0.01] <- "**"
  sig_levels[object$p.values <= 0.001] <- "***"
  
  cat("GARCHX Model Summary:\n\n")
  
  #cat("\nGARCH Component is GARCH(",p,", ",q,")\n\n", sep="")

  cat("GARCH Coefficients:\n")
  alpha_names <- rep("alpha", p)
  for (i in 1:p){
    alpha_names[i] <- paste("alpha_", i, sep="")
  }
  if (q > 0) {
    beta_names <- rep("beta", q)
    for (i in 1:q){
      beta_names[i] <- paste("beta_", i, sep="")
    }
    name <- c("omega", alpha_names, beta_names)
    value = c(object$omega, object$alpha, object$beta)
  } else {
    name <- c("omega", alpha_names)
    value = c(object$omega, object$alpha)
  }

  GARCH_parameter_df <- data.frame(
    name = name,
    value = value,
    pvalue = object$p.values[1:(1+p+q)],
    siglevel = sig_levels[1:(1+p+q)]
  )
  colnames(GARCH_parameter_df)[ncol(GARCH_parameter_df)] <- ""
  print(GARCH_parameter_df, row.names=FALSE)

  if (object$d > 0) ## there are covariates
  {
  
    cat("\nExogenous Covariates Coefficients:\n")
    gamma_names <- rep("gamma", length(object$gamma))
    for (i in 1:length(object$gamma)){
      gamma_names[i] <- paste("gamma_", covariates[i], sep="")
    }
    Data_coefficient_df <- data.frame(
      name = gamma_names,
      value = object$gamma,
      pvalue = object$p.values[(1+p+q+1):length(object$p.values)],
      siglevel = sig_levels[(1+p+q+1):length(sig_levels)]
    )  
    colnames(Data_coefficient_df)[ncol(Data_coefficient_df)] <- ""
    print(Data_coefficient_df, row.names=FALSE)  
  }
  cat("---\n")
  cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n")  
  
  
  cat("\nInformation Criteria:\n")
  cat("  AIC:", AIC(object),"\n")
  cat("  BIC:", BIC(object),"\n")

}

new_GARCHX <- function(eps, X, order = c(1, 1), delta = 2, covariates = "all", optim.method = "NR"){
  
  # Fit the model and returns all the p-values
  
  # Declaring initial values for paramters
  # d = number of exogenous covariates
  if (is.null(X)) {
    d = 0
    covariates = NULL
  } else
  {
    d <- ncol(X)
  }
  r <- 10
  
  
  
  if (!identical(covariates,"all")) {
    if ((covariates[1] == 0) || is.null(covariates)) {
      covariates = NULL
      #return(ARCH_internal(eps=eps, p=order[1], delta=delta))
    } else {
      X <- as.matrix(X[,covariates])
      d = length(covariates)
    }
  } else { #covariates = "all" so change to a vector
    covariates <- 1:ncol(X)
  }
  
  
  # Create object of class GARCHX
  
  obj <- NULL
  obj$covariates <- covariates
  obj$eps <- eps
  obj$selectedX <- X
  obj$X <- X
  obj$N <- length(eps)
  obj$order <- order
  obj$optim.method <- optim.method
  obj$d <- d
  obj$p <- order[1]
  obj$q <- order[2]
  obj$delta <- delta
  obj$r <- r
  obj$alpha.level <- NA
  obj$adjust.method <- 'none'
  obj$omega <- NA     # Initialize omega, alpha, beta, and gamma coefficients as NA
  obj$alpha <- NA
  obj$beta <- NA
  obj$gamma <- NA
  obj$p.values <- NA
  obj$covariate_p.values <- NA
  obj$Likelihood <- NA
  obj$log_QL <- NA
  obj$call <- match.call()   # Assign class to object
  class(obj) <- "GARCHX"
  
  
  # Extract p and q from order
  p <- order[1]
  q <- order[2]
  
  
  # Estimate coefficients
  estimated_values <- GARCHX_estimate(obj, optim.method)
  
  # Extract all estimated coefficients
  estimated_coefs <- estimated_values$coef
  
  # Adjust and assign p-values
  #obj$p.values <- p.adjust(estimated_values$pval[(p+q+2):(1+p+q+d)], method = obj$adjust.method)
  
  # Assign coefficients
  obj$omega <- estimated_coefs[1]
  obj$alpha <- estimated_coefs[2:(p+1)]
  if (q > 0)
    obj$beta <- estimated_coefs[(p+2):(p+q+1)]
  if (d > 0)
    obj$gamma <- estimated_coefs[(p+q+2):(1+p+q+d)]
  
  #if (d > 0)
  #{
  #  obj$selectedX_index <- 1:d # there is no covariate selection, this function is only estimation for all
    obj$p.values <- estimated_values$pval
  #}
  # if covariates are present, set covariate p-values
  if (d > 0) {
    obj$covariate_p.values <- estimated_values$pval[(p + q + 2):(p + q + d + 1)]
  } else {
    obj$covariate_p.values <- NULL ## numeric(0)
  }
  
  # 1/30/26 get covariate p-values
  #if (is.null(covariates)) {
  #  obj$covariate_p_values = NULL
  #} else {
  #  n_covariates <- length(covariates)
  #  obj$covariate_p_values <- obj$p.values[(length(obj$p.values) - n_covariates + 1):(length(obj$p.values))]
  #}
  
  # Assign estimated likelihood
  obj$Likelihood <- estimated_values$minimum
  obj$log_QL <- -1 * obj$Likelihood
  
  # Return obj of GARCHX
  return(obj)
} # End GARCHX.default

GARCHX_likelihood <- function(fit, sigma.init, n, eps.delta, null = FALSE){
  # Assign omega, alpha, beta, and gamma coefficients values
  omega <- fit$omega
  alpha <- fit$alpha
  beta <- fit$beta
  gamma <- fit$gamma

  # Initialize h
  h <- rep(0, n)

  # h^{1/δ_0}_1
  h[1] <- sigma.init^{fit$delta/2}

  # Convert boolean to number
  w <- as.numeric(!null)

  # ht=ω0+ qi=1 α0i+(ε+t−i)δ0+α0i−(ε− t−i)δ0 + pj=1β0jht−j+π′0xt−1
  for(t in 2:n){
    h[t] <- omega + alpha * eps.delta[t-1] + beta * h[t-1] + w * drop(fit$selectedX[t-1,] %*% gamma)
  } # End for

  # Calculate sigma^2
  sigma <- h^(2/fit$delta)

  # Calculate QMLE
  qml <- mean(log(sigma[(fit$r+1):n]) + fit$eps[(fit$r+1):n]**2/sigma[(fit$r+1):n])

  # Return QMLE
  qml
} # End Likelihood.GARCHXselect

GARCHX_estimate <- function(fit, optim.method){
  # fit has the following attributes:
  # eps, X
  # p, q, d
  # omega, alpha, beta, gamma, delta, r

  # Calculate length of time series data
  n <- length(fit$eps)

  # Assign initial values for omega, alpha, beta, and gamma
  fit$alpha <- rep(0.05, fit$order[1])
  # treat when ARCH model for q = 0 and X is NULL
  if (fit$order[2] == 0) {
    fit$beta = NULL
  } else {
    fit$beta = rep(0.05, fit$order[2])
  }
  if (fit$d == 0) {
    fit$gamma = NULL
  } else {
    fit$gamma = rep(1, fit$d)
  }
    
  # Store coefficients in a list for minimization ## ARCH coef alpha are always present
  
  valinit <- c(0.05, fit$alpha, fit$beta, fit$gamma)

  # Initialize Sigma^delta
  sigma.init <- mean(fit$eps[1:fit$r]^fit$delta)

  # Calculate epsilon^delta
  eps.delta <- fit$eps^fit$delta

  # Optimization Function Calls
  if(optim.method == "NR"){
    res <- stats::nlminb(valinit, GARCHX_objf, lower = c(0.0000001, rep(0, sum(fit$order)), rep(0, fit$d)),
                  upper = c(100, rep(0.9999, sum(fit$order)), rep(100, fit$d)),
                  delta = fit$delta, order = fit$order, d = fit$d, r = fit$r, eps = fit$eps,
                  n = n, X = fit$selectedX, eps.delta = eps.delta, sigma.init = sigma.init
                  )
    objective_value <- res$objective
  } else if(optim.method %in% c("L-BFGS-B")){
    # optim does minimization by default
    # upper and lower is only for L-BFGS-B and Brent
    res <- stats::optim(par = valinit, fn = GARCHX_objf, delta = fit$delta, order = fit$order, d = fit$d, r = fit$r, eps = fit$eps,
                 n = n, X = fit$selectedX, eps.delta = eps.delta, sigma.init = sigma.init,
                 method = optim.method,
                 lower = c(0.0000001, rep(0, sum(fit$order)), rep(0, fit$d)),
                 upper = c(100, rep(0.9999, sum(fit$order)), rep(100, fit$d)))
    objective_value <- res$value
  } else if(optim.method == "GA"){
    fitness_fn <- function(coefs){
      -GARCHX_objf(coefs, delta = fit$delta, order = fit$order, d = fit$d, r = fit$r, eps = fit$eps,
            n = n, X = fit$selectedX, eps.delta = eps.delta, sigma.init = sigma.init
            )
    } # End fitness_fn
    sol <- GA::ga(type = "real-valued", fitness = fitness_fn,
              lower = c(0.0000001, rep(0, sum(fit$order)), rep(0, fit$d)),
              upper = c(100, rep(0.9999, sum(fit$order)), rep(100, fit$d)),
              maxiter = 100, monitor = FALSE)
    res <- list(par = sol@solution)
    objective_value <- -sol@fitnessValue
  } else if(optim.method == "SA"){
    res <- GenSA::GenSA(par = valinit, fn = GARCHX_objf,
                 lower = c(0.0000001, rep(0, sum(fit$order)), rep(0, fit$d)),
                 upper = c(100, rep(0.9999, sum(fit$order)), rep(100, fit$d)),
                 delta = fit$delta, order = fit$order, d = fit$d, r = fit$r,
                 eps = fit$eps, n = n, X = fit$selectedX, eps.delta = eps.delta,
                 sigma.init = sigma.init)
    objective_value <- res$value
  } else if(optim.method == "PS"){
    res <- pso::psoptim(par = valinit, fn = GARCHX_objf, gr = NULL, delta = fit$delta,
                   order = fit$order, d = fit$d, r = fit$r, eps = fit$eps, n = n,
                   X = fit$selectedX, eps.delta = eps.delta,
                   sigma.init = sigma.init,
                   lower = c(0.0000001, rep(0, sum(fit$order)), rep(0, fit$d)),
                   upper = c(100, rep(0.9999, sum(fit$order)), rep(100, fit$d)))
    objective_value <- res$value
  } # End if elses

  # Extract coefficients
  p <- fit$order[1]
  q <- fit$order[2]
  d <- fit$d
  omega <- res$par[1]
  alpha <- res$par[2:(p+1)]
  if(q != 0){
    beta <- res$par[(p+2):(p+q+1)]
  } else {
    beta <- NULL
  } # End if


  if(d != 0){
    gamma <- res$par[(p+q+2):(1+p+q+fit$d)]
  } else {
    gamma <- NULL
  } # End if

  # Calculate variance of GARCHX
  varasymp <- GARCHX_variance(fit, omega, alpha, beta, gamma)

  # Return list of coef, sd, t.ratio, pval, Sigma, I matrix, J matrix, and minimum
  list(coef = res$par, sd = varasymp$sd, t.ratio = varasymp$t.ratio,
       pval = varasymp$pval, Sigma = varasymp$Sigma, I = varasymp$I,
       J = varasymp$J, minimum = objective_value)
} # End estimate.GARCHXselect

setting_seed <- function(seed){
  # From R source code for lm simulate --> stats:::simulate.lm
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
    runif(1)
  if (is.null(seed)) 
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  } # set seed
  RNGstate
}

GARCHX_check_gamma <- function(d, t, X){
  # Used in GARCHX_variance to find X.entry

  # Checks if d is 0
  if(d == 0){
    # If yes, val is null
    val <- NULL
  } else {
    # If not, assign the row of exogenous covariates to val
    val <- X[(t-1),]
  } # End if-else

  # Return val
  return(val)
} # End check_pi

sim.GARCHX <- function(n, omega, alpha, beta, gamma, X,  delta = 2, shock.distr = "Normal", valinit = 200){
  #if(!is.matrix(X)){
  #  X <- as.matrix(X)
  #} # End if
  d <- length(gamma)
  
  # # Minimum of alpha, beta, and gamma
  # n2 <- n + max(length(alpha), length(beta)) + 1
  
  if(dim(X)[1] != (n +  valinit)){
    stop("Number of rows of X should be n + valinit")
  } # End if
  
  if(any(is.na(alpha)) || is.null(alpha)){
    alpha = 0
  } # End if
  
  if(any(is.na(beta)) || is.null(beta)){
    beta = 0
  } # End if
  
  if(length(gamma) != dim(X)[2]){
    stop("Dimension of X must match dimension of gamma")
  } #
  
  if (!(shock.distr %in% c("Normal", "t5", "t7", "Laplace"))){
    stop("shock.distr must be Normal, t5, t7, or Laplace")
  } #
  
  # Create eps: Time series
  #####################################################################################################################################
  
  # Calculate how many exogenous covariates
  num_param <- max(length(alpha), length(beta), length(gamma))
  p <- length(alpha)
  q <- length(beta)
  
  #valinit <- valinit - max(length(alpha), length(beta), length(gamma))
  
  # Simulate random white noise
  if (shock.distr == "Normal")
    eta <- rnorm(n+valinit)
  else if (shock.distr == "t5")
    eta <- rt(n+valinit,df = 5)
  else if (shock.distr == "t7")
    eta <- rt(n+valinit,df = 7)

  # Initialize vectors h, sigma, and eps as vectors of 0s
  h <- rep(0, n+valinit)
  sigma <- h
  eps <- h
  
  # Equation #2 in Francq and Thieu
  # ε_t = h^{1/δ_0}_t * η_t
  # For 1:d
  h[1:num_param] <- omega
  # h^{1/δ_0}_t
  sigma[1:num_param] <- h[1:num_param]^(1/delta)
  # Product of h^{1/δ_0}_t and η_t, which becomes ε_t
  eps[1:num_param] <- sigma[1:num_param] * eta[1:num_param]
  
  # Equation #2 in Francq and Thieu
  #  ht=ω0+ qi=1 α0i+(ε+t−i)δ0+α0i−(ε− t−i)δ0 + pj=1β0jht−j+π′0xt−1
  for(t in (num_param+1):(n+valinit)){
    h[t] <- omega + sum(alpha * (eps[(t-1):(t-p)])^delta) + sum(beta * h[(t-1):(t-q)]) + drop(t(gamma) %*% X[t-1,])
    sigma[t] <- h[t]^(1/delta)
    eps[t] <- sigma[t] * eta[t]
  } # End for
  
  
  
  
  
  # Create labeled list as a return value
  i <- valinit + 1
  j <- valinit + n
  
  eps <- eps[i:j]
  X <- X[i:j,]
  #eps <- eps[(d+1):(d+n)]
  
  sim_data <- list(eps = eps, X = X)
  
  return(sim_data)
} # End simulate.GARCHXselect

predict.GARCHX <- function(object, newdata, ...){
  # Extract coefficient values of omega, alpha, beta, and gamma
  omega <- object$omega
  alpha <- object$alpha
  beta <- object$beta
  gamma <- object$gamma
  p <- length(alpha)
  q <- length(beta)

  # ### First Way ###
  # Requires parameters: object, new_X, n_pred only
  eps <- object$eps
  n_pred = dim(newdata)[1]
  # Error check
  #if(dim(X)[1] != n_pred){
  #  stop("Number of rows for X must equal n_pred")
  #} # End if

  # Extract exogenous covariates that were selected by the object
  # newdata <- newdata[, object$selectedX_index]
  # newdata <- rbind(object$selectedX, as.matrix(newdata, ncol = length(object$selectedX_index)))
  newdata <- newdata[, object$covariates, drop=FALSE] # Replace 20-21 above with 22-23 because new_GARCHX no longer has selectedX_index - [AI], 4/2/26
  newdata <- rbind(object$selectedX, as.matrix(newdata, ncol = length(object$selectedX_index)))
  # ### END ###



  # Initialize h as a vector of zeros with size n_pred
  h <- rep(0, n_pred)

  # Calculate the current time by finding length of time series
  current_time <- length(eps)

  t = 1
  if(length(gamma) == 1 && is.na(gamma))
  {

    sigma2_t <- omega + alpha * eps[current_time - 1] + beta * eps[current_time - 1]

    h[1] <- max(0, omega + alpha * eps[current_time] + beta * sigma2_t)

    for(t in 2:n_pred){
      h[t] <- max(0.01, omega + sum(alpha * h[(t-1):(t-p)]) + sum(beta * h[(t-1):(t-q)]))
    } # End for
  } else 
  {
    sigma2_t <- omega + alpha * eps[current_time - 1] + beta * eps[current_time - 1] + drop(newdata[current_time-1,] %*% gamma)

    h[1] <- max(0, omega + alpha * eps[current_time] + beta * sigma2_t + drop(newdata[current_time,] %*% gamma))

    for(t in 2:n_pred)
      h[t] <- max(0.01, omega + sum(alpha * h[(t-1):(t-p)]) + sum(beta * h[(t-1):(t-q)]) + drop(newdata[current_time + t - 1, ] %*% gamma))
  } # End if


  # Return sigma^2
  return(h)
} # End predict.GARCHX

GARCHX_Inv <- function(sigma, tol = sqrt(.Machine$double.eps)){
  # Calculate number of rows in sigma
  n <- nrow(sigma)

  if (n == 1){
    res <- 1/as.numeric(max(sigma, tol))
  } else {
    sp <- eigen(sigma)

    res <- sp$vector %*% diag(1/pmax(abs(sp$values), tol)) %*% t(sp$vector)
  }

  # Return res
  res
} # End Inv

# --- Gaussian kernel between two sets (rows = observations) -------------
# Returns an n_new x n_train matrix with K_ij = exp(-0.5 * ||x_new_i - x_train_j||^2 / h^2)
ARCHmX_kernel_between_iso <- function(Xnew, Xtrain, h) {
  Sa <- rowSums(Xnew^2)                 # length n_new
  Sb <- rowSums(Xtrain^2)               # length n_train
  D2 <- outer(Sa, Sb, "+") - 2 * tcrossprod(Xnew, Xtrain)
  exp(-0.5 * D2 / (h^2))
}

new_ARCHmX <- function(eps, X, p, delta=2, covariates="all", hypothesisTest=TRUE, bandwidth = "plugin", k.choice = "plugin") {
  

  #################################################
  # A: Fit the model
  #################################################
  
  # 10/27: Rewritten to use professor's matrix routines from simulation5covariatesNormal
  
  if ((!identical(covariates,"all")) || (is.null(X))) {
    if ((((covariates[1] == 0) || is.null(covariates))) || (is.null(X))) {
      covariates = NULL
      #return(ARCH_internal(eps=eps, p=p, delta=delta))
      return(GARCHX(eps=eps, X = NULL, order = c(p,0), delta=delta))
    } else {
      X <- as.matrix(X[,covariates])
    }
  } else { #covariates = "all" so change to a vector
    covariates <- 1:ncol(X)
  }
  
  get_W <- function(X, optimal_h) {
    Kernel_matrix  = ARCHmX_kernel_matrix_iso(X,optimal_h)
    row_normalize <- function(K) {
      K / pmax(rowSums(K), .Machine$double.eps)
    }
    W <- row_normalize(Kernel_matrix)
    W = W[(p):(N-1),(p):(N-1)]
    return(W)
  }
  
  get_epsilon_matrix <- function(epsilon_delta) {
    Epsilon_matrix = matrix(0,N-p,p)
    for (j in 1:p) {
      Epsilon_matrix[,j] = epsilon_delta[(p-j+1):(N-j)] 
    }
    return(Epsilon_matrix)
  }
  
  get_X_tilde <- function(W, Epsilon_matrix) {
    X_tilde = matrix(0,N-p,p)
    X_tilde = Epsilon_matrix - W%*%Epsilon_matrix
    return(X_tilde)
  }
  
  N <- length(eps)
  epsilon_delta <- eps^delta
  if (bandwidth == "plugin") {
    s_x <- sd(X[,1], na.rm=TRUE)
    optimal_h <- s_x * N^(-1/5)
  } else{
    h_grid = seq(0.01, (range(X[,1])/4)[2], length.out = 10)
    #optimal_h <- ARCHmX_nw_cv_bandwidth(matrix(X[p:(N-1),], nrow=N-p), epsilon_delta[(p+1):N], h_grid = h_grid, nfolds = 10)$best_h
    optimal_h <- ARCHmX_nw_cv_bandwidth(matrix(X, nrow=nrow(X)), epsilon_delta, h_grid = h_grid, nfolds = 10)$best_h
  }
  Epsilon_matrix <- get_epsilon_matrix(epsilon_delta)
  W <- get_W(X, optimal_h)
  Y_tilde = epsilon_delta[(p+1):N] - as.vector(W %*% epsilon_delta[(p+1):N])  
  X_tilde <- get_X_tilde(W, Epsilon_matrix) 
  estim_alpha = nnls(X_tilde, Y_tilde)$x
  unexplained_by_ARCH  <- epsilon_delta[(p+1):N] - as.vector(Epsilon_matrix %*% estim_alpha)   # {p+1} through T, length T-p
  m_hat <- as.vector(W %*% unexplained_by_ARCH)                                                 # M(X_{p}) through M(X_{T-1}), length T-p
  # originally v_hat was computed here as unexplained_by_ARCH - m_hat per paperEJS but
  # this has been modified (here and below in the hypothesis test section) to match professor's code
  
  #################################################
  # B: Get Performance Metrics 
  #################################################
  
  get_sigma_tilde_2 <- function(estim_alpha, epsilon_delta, m_hat) {
    m_hat = c(rep(0,p),m_hat)
    sigma_tilde_d <- rep(0, N)
    sigma_tilde_d.init <- mean(epsilon_delta[1:10])    # r hardcoded to 10 based on line 665 simulation5covariatesNormal.r
    sigma_tilde_d[1:p]<-sigma_tilde_d.init 
    for (t in (p+1):N) 
    {
      # original code just omits epsilons prior to p+1, m_hat index aligned with sigma_tilde (seems 
      # right based off the documentation but Professor believes there is an indexing issue here)
      #  sigma_tilde_d[i] <- sum(estim_alpha * epsilon_delta[(p+i-1):i]) + max(0,m_hat[i])
      sigma_tilde_d[t] <- estim_alpha%*%(epsilon_delta[(t-1):(t-p)]) + max(0,m_hat[t-1])
    }    
    return(sigma_tilde_d^(2/delta))
  }
  
  sigma_tilde_2 <- get_sigma_tilde_2(estim_alpha, epsilon_delta, m_hat)
  sigma_tilde_2[which(sigma_tilde_2 == 0)] = mean(sigma_tilde_2)
  #v_hat <- (eps^2)[(p+1):N] - sigma_tilde_2
  v_hat <- (eps^2) - sigma_tilde_2
  #v_hat[which(is.nan(v_hat))] = 0
  v_hat[!is.finite(v_hat) | is.na(v_hat)] = 0 
  residual_deviation <- sqrt(mean(v_hat^2))
  
  #log_QL <- ( log(2 * pi) * -(N-p)/2 ) - sum(log(sigma_tilde_2) / 2) - sum( (eps^2)[(p+1):N] / sigma_tilde_2 ) / 2
  log_QL <- ( log(2 * pi) * -(N-p)/2 ) - sum(log(sigma_tilde_2) / 2) - sum( (eps^2) / sigma_tilde_2 ) / 2
  k <- sum(diag(W))
  
  df <- k + p
  AIC <- 2 * df - 2 * log_QL
  BIC <- df * log(N-p) - 2 * log_QL
  
  #################################################
  # C: Hypothesis testing for variable significance  
  #################################################
  
  alpha_p.values <- rep(NA_real_, p) # TBD!!
  covariate_p.values <- NULL
  p.values <- NULL
  
  if (hypothesisTest) {
    # Step B1: Get the v_hats
    
    # get the v_hats in a matrix, each row corresponds to the residuals for a given covariate
    # the ij'th entry corresponds to (p+j-1)th entry of the i'th covariate. In other words, 
    # v_hats_{ij} goes with X_{p+j-1, i}
    
    n_vars <- ncol(X)
    v_hats <- c()
    for (i in 1:n_vars) 
    {
      X_prime <- matrix(X[, -i], nrow=nrow(X))
      if (n_vars > 1) #(length(X_prime) > 0)    ## It means that there is only one covariate and when removing there are no covariates left
      {
        if (bandwidth == "plugin") {
          s_x_prime <- sd(X_prime[,1], na.rm=TRUE)
          optimal_h_prime <- s_x_prime * N^(-1/5)
        } else{
          h_grid_prime = seq(0.01, (range(X_prime[,1])/4)[2], length.out = 10)
          #optimal_h_prime <- ARCHmX_nw_cv_bandwidth(matrix(X_prime[p:(N-1),], nrow=N-p), epsilon_delta[(p+1):N], h_grid = h_grid_prime, nfolds = 10)$best_h
          optimal_h_prime <- ARCHmX_nw_cv_bandwidth(matrix(X_prime, nrow=nrow(X_prime)), epsilon_delta, h_grid = h_grid_prime, nfolds = 10)$best_h
        }
        
        W_prime <- get_W(X_prime, optimal_h_prime)
        Y_tilde_prime = epsilon_delta[(p+1):N] - as.vector(W_prime %*% epsilon_delta[(p+1):N])  
        X_tilde_prime <- get_X_tilde(W_prime, Epsilon_matrix)
        estim_alpha_prime = nnls(X_tilde_prime, Y_tilde_prime)$x
        unexplained_by_ARCH_prime <- epsilon_delta[(p+1):N] - as.vector(Epsilon_matrix %*% estim_alpha_prime) 
        m_hat_prime <- as.vector(W_prime %*% unexplained_by_ARCH_prime)  
        #v_hat_prime <- unexplained_by_ARCH_prime - m_hat_prime # commented 12/14 - see below
        #v_hat_prime <- (eps^2)[(p+1):N] - sigma_tilde_2_prime
      }
      else  # no covariates
      { 
        #empty_model <- ARCH_internal(eps, p, delta = delta)
        empty_model <- GARCHX(eps=eps, X = NULL, order = c(p,0), delta=delta)
        estim_alpha_prime = empty_model$alpha
        m_hat_prime = rep(0,length(epsilon_delta))
      }
      sigma_tilde_2_prime <- get_sigma_tilde_2(estim_alpha_prime, epsilon_delta, m_hat_prime)
      v_hat_prime <- (eps^2) - sigma_tilde_2_prime
      v_hat_prime[which(is.nan(v_hat_prime))] = 0
      v_hats <- c(v_hats, v_hat_prime)
    }
    
    # Step B2: Perform the tests
    B <- 200
    alpha <- 0.05
    covariate_p.values <- rep(0, n_vars)
    #n <- ncol(v_hats) # in paper_EJS, n is the number of observation-residual pairs for the given covariate
    
    next_odd <- function(n) {
      n <- floor(n)                 # floor each element
      ifelse(n %% 2 == 0, n + 1, n) # if even → next odd; if odd → keep
    }
    range <- unique(next_odd(seq(next_odd(N^(1/3)/2), next_odd(N^(1/3)), length = 5)))
    
    for (i in 1:n_vars)
    {
      #X_values <- X[,i][p:(N - 1)]
      #Y_values <- v_hats[i,]
      # Y_values <- v_hats[i,][2:N]
      #k_n <- ARCHmX_choosek(X_values, Y_values, range, B, alpha)
      
      X_values <- X[,i][1:(N - 1)]
      offset <- N * (i-1)
      Y_values <- v_hats[(offset + 2):(offset + N)]
      if (k.choice == "plugin")
        k_n = next_odd(N^(29/90))
      else
        k_n <- ARCHmX_choosek(X_values, Y_values, range = range, B = B, alpha = 0.05)
      
      covariate_p.values[i] <- ARCHmX_anova_test_univariate(X_values, Y_values, k_n)
    }
  }
  
  
  if (is.null(covariate_p.values)) {
    covariate_p.values <- numeric(0)
  }
  
  p.values <- c(alpha_p.values, covariate_p.values)

  #################################################
  # D: Set model parameters 
  #################################################

  structure(
    list(
      h = optimal_h,
      N = N,
      covariates = covariates,
      delta = delta, #
      df = df,
      log_QL = log_QL,
      epsilon_delta = epsilon_delta,
      p = p,
      q = 0,
      X = X, #
      eps = eps,
      alphas = c(estim_alpha), #
      p.values = c(p.values),
      covariate_p.values = c(covariate_p.values),
      m_hat = m_hat, #
      v_hat = v_hat,
      residual_deviation = residual_deviation,
      AIC = AIC,
      BIC = BIC,
      hypothesisTest = hypothesisTest,
      bandwidth = bandwidth,
      k.choice = k.choice
    ),
    class = "ARCHmX"
  )
}

AIC.GARCHX <- function(object, ..., k){
  # Calculate initial value for sigma^2
  #sigma.init <- mean(object$eps[1:object$r]^2)
  sigma.init <- mean(object$eps[1:object$r]^object$delta)

  # Calculate epsilon^delta
  eps.delta <- object$eps^object$delta

  # Check if object is GARCHX or GARCH
  if(is.na(object$gamma[1])){
    null = TRUE
  } else {
    null = FALSE
  } # End if-else

  # Calculate QMLE using likelihood
  # QMLE <- Likelihood(object, sigma.init = sigma.init, length(object$eps),
  #                    eps.delta = eps.delta, null)
  # Do we need to use nlminb here?

  p <- object$order[1]
  q <- object$order[2]
  # Using likelihood, calculate AIC value
  AIC <- 2 * length(object$eps) * object$Likelihood + (1 + p + q + object$d) * 2

  # Return AIC value
  return(AIC)
} # End AIC.GARCHX

# Francq, C. and Thieu, L.Q.(2018). *QML Inference for Volatility Models with Covariates*.
GARCHX_variance <- function(fit, omega, alpha, beta, gamma, tol = (.Machine$double.eps)){
  # Extract eps, delta, r and exogenous covariates
  eps <- fit$eps
  delta <- fit$delta
  r <- fit$r
  X <- fit$selectedX 

  # Calculate length of time series
  n <- length(eps)

  # Calculate order of alpha and beta, and length of exogenous covariates
  d <- fit$d
  p <- fit$order[1]
  q <- fit$order[2]
  dim_1 <- 1+p+q+d

  # Initialize h, sigma^2, and eta vectors
  h <- rep(0, n)
  sigma2 <- rep(0, n)
  eta <- rep(0, n)

  # Initialize derivative of h, S, I, and J as zero matrices
  der.h <- matrix(0, nrow = dim_1, ncol = n)
  S <- matrix(0, nrow = dim_1, ncol = n)
  I <- matrix(0, nrow = dim_1, ncol = dim_1)
  J <- matrix(0, nrow = dim_1, ncol = dim_1)

  # Calculate initial value of sigma^delta
  sigma2.init <- mean(eps[1:r]^delta)
  h[1:(max(p, q))] <- sigma2.init^{delta/2}
  eps.delta <- eps^delta
  if(d == 0) {
    null <- TRUE
  } else {
    null = FALSE
  }
  # Calculate derivative of h
  if(null == FALSE)
  {

    for(t in (2+(max(p,q))-1):r){
      # Problem is at GARCHX_check_order with
      # eps.delta[(t-1):(t-p)]
      # If p = 2, we get eps.delta[1:0], which only returns one value
      # But we need it to return two values
      eps.entry <- GARCHX_check_order(p, t, eps.delta)

      # Check if q = 0
      h.entry <- GARCHX_check_order(q, t, h)

      # Check if d = 0
      X.entry <- GARCHX_check_gamma(d, t, X)

      # 1/27 per professor
      #if(q == 1 || q == 0){
      #  vol <- beta * h[(t-1):(t-q)]
      #  temp <- beta * der.h[, (t-1):(t-q)]
      #} else {
      #  vol <- beta %*% h[(t-1):(t-q)]
      #  temp <- der.h[, (t-1):(t-q)] %*% beta
      #}
      
      if (q == 0) 
      {
        vol <- 0
        temp <- 0
      }
      else if (q == 1)
      {
        vol <- beta * h[(t-1):(t-q)]
        temp <- beta * der.h[, (t-1):(t-q)]
      }
      else 
      {
        vol <- beta %*% h[(t-1):(t-q)]
        temp <- der.h[, (t-1):(t-q)] %*% beta
      }

      if (d == 0)
        h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol 
      else
        h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol + drop(X[t-1,] %*% gamma)
      # dim of der.h = 9 x 2
      # dim of beta = 2 x 1
      der.h[ ,t] <- c(1, eps.entry, h.entry, X.entry) + temp
    } # End for

    for (t in (r+1):n){
      eps.entry <- GARCHX_check_order(p, t, eps.delta)
      h.entry <- GARCHX_check_order(q, t, h)

      X.entry <- GARCHX_check_gamma(d, t, X)

      #if(q == 1 || q == 0){
      #  vol <- beta * h[(t-1):(t-q)]
      #  temp <- beta * der.h[, (t-1):(t-q)]
      #} else {
      #  vol <- beta %*% h[(t-1):(t-q)]
      #  temp <- der.h[, (t-1):(t-q)] %*% beta
      #}
      
      if (q == 0) 
      {
        vol <- 0
        temp <- 0
      }
      else if (q == 1)
      {
        vol <- beta * h[(t-1):(t-q)]
        temp <- beta * der.h[, (t-1):(t-q)]
      }
      else 
      {
        vol <- beta %*% h[(t-1):(t-q)]
        temp <- der.h[, (t-1):(t-q)] %*% beta
      }
      
      if (d == 0)
        h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol 
      else
        h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol + drop(X[t-1,] %*% gamma)
      der.h[ ,t] <- c(1, eps.entry, h.entry, X.entry) + temp
      # der.h[ ,t] <- c(1, eps.entry, h.entry, X.entry) + as.matrix(beta) %*% der.h[,(t-1):(t-q)]

      S[, t] <- der.h[,t]/h[t]

      sigma2[t] <- h[t]^(2/delta)

      eta[t] <- eps[t]/max(sqrt(sigma2[t]), tol)

      I <- I + (4/delta^2) * (eta[t]^4-1) * S[,t] %*% t(S[,t])/(n-r)

      J <- J + (4/delta^2) * S[,t] %*% t(S[,t])/n
    } # End for
  } else {

    for(t in (2+(max(p,q))-1):r){
      eps.entry <- GARCHX_check_order(p, t, eps.delta)
      h.entry <- GARCHX_check_order(q, t, h)

      #if(q == 1 || q == 0){
      #  vol <- beta * h[(t-1):(t-q)]
      #  temp <- beta * der.h[, (t-1):(t-q)]
      #} else {
      #  vol <- beta %*% h[(t-1):(t-q)]
      #  temp <- der.h[, (t-1):(t-q)] %*% beta
      #}
      
      if (q == 0) 
      {
        vol <- 0
        temp <- 0
      }
      else if (q == 1)
      {
        vol <- beta * h[(t-1):(t-q)]
        temp <- beta * der.h[, (t-1):(t-q)]
      }
      else 
      {
        vol <- beta %*% h[(t-1):(t-q)]
        temp <- der.h[, (t-1):(t-q)] %*% beta
      }

      h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol

      der.h[,t] <- c(1, eps.entry, h.entry) +  temp
    } # End for

    for(t in (r+1):n){
      eps.entry <- GARCHX_check_order(p, t, eps.delta)
      h.entry <- GARCHX_check_order(q, t, h)
      #if(q == 1 || q == 0){
      #  vol <- beta * h[(t-1):(t-q)]
      #  temp <- beta * der.h[, (t-1):(t-q)]
      #} else {
      #  vol <- beta %*% h[(t-1):(t-q)]
      #  temp <- der.h[, (t-1):(t-q)] %*% beta
      #}
      if (q == 0) 
      {
        vol <- 0
        temp <- 0
      }
      else if (q == 1)
      {
        vol <- beta * h[(t-1):(t-q)]
        temp <- beta * der.h[, (t-1):(t-q)]
      }
      else 
      {
        vol <- beta %*% h[(t-1):(t-q)]
        temp <- der.h[, (t-1):(t-q)] %*% beta
      }

      h[t] <- omega + alpha %*% eps.delta[(t-1):(t-p)] + vol

      der.h[,t] <- c(1, eps.entry, h.entry) +  temp

      S[,t] <- der.h[,t]/h[t]

      sigma2[t] <- h[t]^(2/delta)

      eta[t] <- eps[t]/max(sqrt(sigma2[t]), tol)

      I <- I + (4/delta^2) * (eta[t]^4-1) * S[,t] %*% t(S[,t])/(n-r)

      J <- J + (4/delta^2) * S[,t] %*% t(S[,t])/n
    } # End for
  } # End if-else

  mu4 <- mean(eta[(r+1):n]^4)
  mu2 <- mean(eta[(r+1):n]^2)

  Jinv <- GARCHX_Inv(J)

  Sigma.iid <- (mu4 - mu2) * Jinv

  Sigma <- Jinv %*% I %*% Jinv

  if(min(eigen(Sigma)$values) < tol){
    Sigma <- Sigma.iid
  } # End if

  sd <- sqrt(diag(Sigma/(n-r)))

  sd.iid <- sqrt(diag(Sigma.iid/n))

  t.ratio <- c(omega, alpha, beta, gamma)/sd

  t.ratio.iid <- c(omega, alpha, beta, gamma)/sd.iid

  pval <- (1 - pchisq(t.ratio^2, 1))/2

  pval.iid <- (1 -pchisq(t.ratio.iid^2, 1))/2

  list(sd = sd, pval = pval, sd.iid = sd.iid, pval.iid = pval.iid,
       t.ratio = t.ratio, t.ratio.iid = t.ratio.iid, Sigma = Sigma,
       Sigma.iid = Sigma.iid, I = I, J = J)
} # End variance.GARCHX

# X: n x d matrix (each row = x_i)
# h: scalar bandwidth
# returns a matrix of kernels K_ij for each entry (xi-xj)
ARCHmX_kernel_matrix_iso <- function(X, h) {
  s  <- rowSums(X^2)                         # length-n
  D2 <- outer(s, s, "+") - 2 * tcrossprod(X) # pairwise ||x_i - x_j||^2
  exp(-0.5 * D2 / (h^2))                     # unnormalized Gaussian kernel
  # If you want the normalized density kernel, multiply by:
  # 1 / ((2*pi)^(ncol(X)/2) * h^ncol(X))
}

predict.ARCHmX <- function(object, newdata, ...){
  # strip newdata to only the columns used in object
  
  if (is.null(object$covariates)) {
    stop("Predict Function requires an ARCHmX object with a data component.")
  } else if (p == 0) {
    stop("Prediction requires p > 0.")
  }
  
  # newdata <- matrix(rnorm(100), nrow=10)
  newdata <- matrix(newdata[, object$covariates], nrow=nrow(newdata))
  
  # retrieve the necessary attributes from the model object
  
  h <- object$h
  epsilon_delta <- object$epsilon_delta
  p <- object$p
  alphas <- matrix(object$alphas, ncol=1)
  T_ <- object$T_
  train <- matrix(object$X[, object$covariates], nrow=nrow(object$X))[(p+1):T_,]
  delta <- object$delta
  
  # t-1 correction:
  #   Since epsilon^delta_t depends on the data at time t-1, our "newdata" should
  #   consist of the time T_ data along with everything but the last new observation.
  
  newdata <- rbind(matrix(train[nrow(train),], nrow=1), matrix(newdata[(1:(nrow(newdata)-1)),], ncol=ncol(newdata)))
  
  # build the epsilon matrix
  
  get_epsilon_matrix <- function(epsilon_delta) {
    Epsilon_matrix = matrix(0,T_-p,p)
    for (j in 1:p) {
      Epsilon_matrix[,j] = epsilon_delta[(p-j+1):(T_-j)] 
    }
    return(Epsilon_matrix)
  }
  Epsilon_matrix <- get_epsilon_matrix(epsilon_delta)
  
  # build the weight matrix 
  
  W <- ARCHmX_kernel_between_iso(newdata, train, h)
  
  # Generate the m_hat matrix
  
  m_hats <- W %*% (matrix(epsilon_delta[(p+1):T_], ncol=1) - Epsilon_matrix %*% alphas)
  
  # Expand epsilon_delta and generate the new predicted epsilon_deltas 
  
  epsilon_delta <- c(epsilon_delta, rep(0, nrow(newdata)))
  #for i in ((T_+1):(T_+nrow(newdata))) {
  for (i in 1:nrow(m_hats)) {
    epsilon_delta[T_ + i] <- epsilon_delta[(T_+i-1):(T_+i-p)] %*% alphas + max(m_hats[i],0)
  }
  
  # return the new predicted volatilities
  
  return(epsilon_delta[(T_+1):(T_+nrow(m_hats))]^(2/delta))
} # End predict.ARCHmX

stepwise <- function(target_model, direction="forward", criterion="pvalue", adjust_method = "fdr", alpha.level=0.05, trace=TRUE) {
  # Validation
  if (!(direction %in% c("forward", "backward", "both_null", "both_full", "bestsubset", "onepass"))) {
    stop("Error: direction must be 'forward', 'backward', 'both_null', 'both_full', 'bestsubset', or 'onepass'.")
  }
  if (!(criterion %in% c("pvalue", "AIC", "BIC"))) {
    stop("Error: criterion must be 'pvalue', 'AIC', or 'BIC'")
  }
  if (!(adjust_method %in% c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none"))) {
    stop("Error: adjust_method must be 'holm', 'hochberg', 'hommel', 'bonferroni', 'BH', 'BY', 'fdr', or 'none'.")
  } 
  if (is.null(target_model$covariates) || is.null(target_model$X)) {
    stop("Error: Target model must have covariates and a data matrix.")
  }
  if (ncol(target_model$X) != length(target_model$covariates)) {
    stop("Error: Target model must incorporate all covariates.")
  }
  if (is(target_model, "ARCHmX") || is(target_model, "ARCHmXAdditive")) {
    if (target_model$hypothesisTest == FALSE) {
      stop("Error: Target model requires hypothesis test.")
    }
  }
  if ((!is(target_model, "GARCHX")) && (!is(target_model, "ARCHmX"))  && (!is(target_model, "ARCHmXAdditive"))) {
    stop("Error: This function can only be called on GARCHX, ARCHmX, and ARCHmXAdditive objects.")
  }

  # Steps taken structure
  steps_taken <- structure(
    list(
      covariates = c(),
      aic = c(),
      bic = c(),
      maxpval = c(),
      current_model = list()
    )
  )
  
  generate_candidate_model <- function(target_model, covariates) {
    if (is(target_model, "GARCHX")) {
      return(new_GARCHX(eps = target_model$eps,
                        X = target_model$X,
                        order = target_model$order,
                        delta = target_model$delta,
                        covariates = covariates,
                        optim.method = target_model$optim.method
                        ))
    } else if (is(target_model, "ARCHmX")) {
      return (new_ARCHmX(X = target_model$X,
                         p = target_model$p,
                         eps = target_model$eps,
                         delta = target_model$delta, 
                         covariates = covariates, 
                         bandwidth = target_model$bandwidth,
                         k.choice = target_model$k.choice
                         ))
    } else if (is(target_model, "ARCHmXAdditive")) {
      return(new_ARCHmXAdditive(X = target_model$X,
                                p = target_model$p,
                                eps = target_model$eps,
                                delta = target_model$delta,
                                covariates = covariates,
                                method = target_model$method,
                                train_end = target_model$train_end,
                                n_burn = target_model$n_burn,
                                max_outer_iter = target_model$max_outer_iter,
                                tol = target_model$tol,
                                hypothesisTest = TRUE,
                                k.choice = target_model$k.choice
                                ))
    }
  }
  
  # Helper functions for main logic below
  
  # rewritten 11/20 to take advantage of new model covariate field
  store_model <- function(model, steps_taken) {
    if (is.null(model$covariate_p.values)) { # empty model
      pvalue <- c("NA")
    } else {
      pvalue <- max(p.adjust(model$covariate_p.values, method = adjust_method))
    }
    new_steps_taken <- structure(
      list(
        covariates = c(steps_taken$covariates, paste(model$covariates, collapse=", ")),
        aic = c(steps_taken$aic, AIC(model)),
        bic = c(steps_taken$bic, BIC(model)),
        maxpval = c(steps_taken$maxpval, pvalue),
        current_model = model
      )
    )
    return(new_steps_taken)
  }
  
  # 11/23 - maybe it might make sense to change how
  # ARCH_internal.R and ARCHmX.R work instead of 
  # using this workaround to get the model covariate count
  covariate_count <- function(model) {
    if (is.null(model$covariates)) { # ARCH(p) "empty" model
      return(0)
    } else {
      return(length(model$covariates))
    }
  }
  
  # reviewed 11/20, i think this is still good
  combinations <- function(current_covariates, size) {
    # returns a matrix of row-wise covariate combinations
    # We only need to handle the forward and backward directions 
    # since the both logic will just call this function twice
    if (is.null(current_covariates)) {
      current_covariates <- c()
    }
    if (length(current_covariates) < size) { # This is the forward direction
      combs <- t(combn(ncol(target_model$X), size))
      for (index in current_covariates) {
        row_contains_index <- apply(combs, MARGIN = 1, FUN = function(row) {index %in% row})
        combs <- matrix(combs[row_contains_index, ], ncol=size)
      }
    } else if (length(current_covariates) > size) { # this is the backward direction
      combs <- t(combn(current_covariates, size)) # size should always be length(sig...) - 1
    }
    return(combs)
  }
  
  # reviewed 11/20, looks good
  # 1/12/26 added functionality to handle common_df scenario. See comments on line 204.
  metric <- function(model, criterion, common_df=FALSE) {
    if(is.null(model$covariates)) 
    {
      offset = 1
    } else {
      offset = 0
    }
    if (criterion == "AIC") 
    {
      if(common_df) {
        df <- model$p + offset
        log_QL <- model$log_QL
        return(2 * df - 2 * log_QL)
      }
      else {
        return(AIC(model))
      }
    }
    else if (criterion == "BIC") {
      if(common_df) {
        p <- model$p
        df <- p + offset
        T_ <- length(model$eps)
        log_QL <- model$log_QL
        return(df * log(T_-p) - 2 * log_QL)
      }
      else {
        return(BIC(model))
      }
    }
    else { # criterion == "pvalue"
      if (!is.null(model$covariate_p.values)) {
        return(max(p.adjust(model$covariate_p.values, method = adjust_method)))
      } else { # empty model
        return(NA) 
      }
    }
  }
  
  # written 11/21, updated 11/26
  step_difference <- function(current, proposed) {
    current_count <- covariate_count(current)
    proposed_count <- covariate_count(proposed)
    if (current_count == proposed_count) {
      return("<none>")
    } else if (current_count < proposed_count) { # forward (additive) step
      for (j in 1:proposed_count) {
        if (!(as.character(proposed$covariates[j]) %in% as.character(current$covariates))) {
          return(paste("+", as.character(proposed$covariates[j])))
        }
      }
    } else { # length(covs1) > length(covs2), backward (reductive) step
      for (i in 1:current_count) {
        if (!(as.character(current$covariates[i]) %in% as.character(proposed$covariates))) {
          return(paste("-", as.character(current$covariates[i])))
        }
      }   
    }
  }
  
  # 1/12/26 helper function to identify comparisons against null model
  contains_null_model <- function(candidates) {
    null_model_found <- FALSE
    for (i in 1:length(candidates)) {
      null_model_found <- null_model_found || is.null(candidates[[i]]$X)
    }
    return(null_model_found)
  }
  
  # written 11/21
  display_step <- function(candidates, ranked_candidate_indices, step_to_index) {
    row_names <- c()
    AIC <- c()
    BIC <- c()
    max_pvalue <- c()
    pass.correction <- c()
    
    # 1/12/26: If candidates include the null model, we need to get an apples-to-apples comparison.
    #          To do so, 'metric' function will treat all models as having df=1+P (what we've 
    #          termed 'common_df') when computing AIC or BIC. 
    common_df <- contains_null_model(candidates)
    
    for (index in ranked_candidate_indices) {
      row_names <- c(row_names, step_difference(candidates[[1]], candidates[[index]]))
      AIC <- c(AIC, metric(candidates[[index]], "AIC", common_df))
      BIC <- c(BIC, metric(candidates[[index]], "BIC", common_df))
      max_pvalue <- c(max_pvalue, metric(candidates[[index]], "pvalue"))
    }
    pass.correction <- (max_pvalue < alpha.level)
    selected <- rep("", length(candidates))
    selected[which(step_to_index == ranked_candidate_indices)] <- "*"
    
    cat("Call:\n")
    cat(class(target_model),"(X, p, eps, delta=", candidates[[1]]$delta, ", covariates=",
        paste(candidates[[1]]$covariates,sep=","),
        ")\n\n", sep="")
    
    cat("Covariates:\n")
    cat(paste(candidates[[1]]$covariates, sep=", "), "\n\n")
    
    step.df <- data.frame(
      #Resid.Dev, AIC, BIC, max_pvalue, pass.correction, selected,
      AIC, BIC, max_pvalue, pass.correction, selected,
      row.names = row_names
    )
    
    print(step.df)
    cat("\n\n")
  }
  
  # rewritten 11/22
  attempt_step <- function(candidates, step_direction="forward") { 
  # can't call the parameter 'direction' because I need to be able to step forward and backward within the 'both' logic
    
    # 1/12/26: If candidates include the null model, we need to get an apples-to-apples comparison.
    #          To do so, 'metric' function will treat all models as having df=1+P (what we've 
    #          termed 'common_df') when computing AIC or BIC. 
    common_df <- contains_null_model(candidates)
    
    max_adj_pval <- rep(0, length(candidates))
    metrics <- rep(0, length(candidates))
    for (i in 1:length(candidates)) {
      # populate the max_adj_pval vector, handling the empty model if it comes up
      # The adjustments below ensure the correct step happens.
      
      # Forward step from empty model to univariate model:
      #     P value metric: Empty model has metric 1 so the prospective models will always improve the metric.
      #                     The step will happen as long as some prospective univariate model has significant p-value. 
      #                     This is exactly what should happen based on the hypothesis test for the univariate model. 
      #   Any other metric: The empty model cannot be a prospective model since it doesn't improve on its own metric.
      #                     So it doesn't matter what its p value is. We can only step to some other model.
      
      # Backward step from univariate model to empty model:
      #     P value metric: Empty model has metric 0 so the step will always happen whenever the univariate model
      #                     has insignificant p-value. This is exactly what should happen based on the hypothesis
      #                     test for the univariate model.
      #   Any other metric: The step will happen only when there is metric improvement. 

      if (is.na(metric(candidates[[i]], "pvalue"))) {
        if (step_direction == "forward") {
          max_adj_pval[i] <- 1
        } else if (step_direction == "backward") {
          max_adj_pval[i] <- 0
        } else {
          # p value cannot be the metric for both selection, 
          # so we are only using it to decide whether we may take forward steps
          # We can only move forward FROM the empty model, so it doesn't matter
          #       what we do with the p-value, because the current doesn't improve the metric
          # We can only move backward TO the empty model, and again it doesn't matter
          #       what we do with the p-value, because backward steps for non-p value metrics
          #       don't care about p value.
          # So, we will just make the p-value 0 to have a way to identify the null model when we find it
          max_adj_pval[i] <- 0
        }
      } else {
        max_adj_pval[i] <- metric(candidates[[i]], "pvalue")
      }
      # populate the metrics vector
      if (criterion == "pvalue") {
        metrics[i] <- max_adj_pval[i]
      } else {
        metrics[i] <- metric(candidates[[i]], criterion, common_df)
      }
    }

    ranked_candidate_indices <- order(metrics) # we want to minimize all metrics, so default ascending order is correct
    metric_improved <- (metrics[ranked_candidate_indices] < metrics[1])
    
    if (step_direction == "forward") {
      
      # We step to the model with most significant metric improvement, which also has all significant adjusted p-values.
      # If none fit the criteria, we stay with the current model.
    
      all_pvals_signif <- (max_adj_pval[ranked_candidate_indices] < alpha.level)
      
      # 12/21: slight modification to force step when metric is p-value and there are candidate models
      # with all significant p-values (as originally written, if min(max(pvalue)) belongs to the current model,
      # step doesn't happen)
      
      if (criterion == "pvalue") {
        better_model_indices <- ranked_candidate_indices[all_pvals_signif]
        better_model_indices <- better_model_indices[which (better_model_indices != 1)]
      } else {
        better_model_indices <- ranked_candidate_indices[metric_improved & all_pvals_signif]
      }
      
      #  better_model_indices <- ranked_candidate_indices[metric_improved & all_pvals_signif]
      
      if (length(better_model_indices) == 0) { # no improved model with all significant p-values
        step_to_index <- 1
      } else {
        step_to_index <- better_model_indices[1]
      }
      
    } else if (step_direction == "backward") {
      
      # We step to the model with the most significant metric improvement,
      # unless the current model has all significant adjusted p-values
      
      better_model_indices <- ranked_candidate_indices[metric_improved]
      
      #if ( (length(better_model_indices) == 0) | (max_adj_pval[1] < alpha.level) ) {
      #  step_to_index <- 1
      #} else {
      #  step_to_index <- better_model_indices[1]
      #}
      
      # if the current model has all significant p-values, stop.
      if (max_adj_pval[1] < alpha.level) {
        step_to_index <- 1
        # if no reduced model outperforms the current model, pick whichever one does the best
      } else if (length(better_model_indices) == 0) {
        step_to_index <- ranked_candidate_indices[2]
        # otherwise, we step to the best overall model
      } else {
        step_to_index <- better_model_indices[1]
      }
      
    } else if (step_direction == "both") {
      # we only care about the p-value check when taking forward steps
      p_value_good <- rep(TRUE, length(ranked_candidate_indices))
      current_cov_count <- covariate_count(candidates[[1]])
      for (i in 1:length(ranked_candidate_indices)) {
        ranked_candidate_cov_count <- covariate_count(candidates[[ranked_candidate_indices[i]]])
        if (ranked_candidate_cov_count > current_cov_count) {
          p_value_good[i] <- (metric(candidates[[ranked_candidate_indices[i]]], "pvalue") < alpha.level)
        } else {
          p_value_good[i] <- TRUE
        }
      }
      
      better_model_indices <- ranked_candidate_indices[metric_improved & p_value_good]
      
      # we don't have to worry about the current model having all significant p-values because
      # in the main body of the both selection logic, we never built backward candidates in that case.
      if ( length(better_model_indices) == 0 ) {
        step_to_index <- 1
      } else {
        step_to_index <- better_model_indices[1]
      }
    }
    
    if (trace) {
      display_step(candidates, ranked_candidate_indices, step_to_index) 
    }
    return(candidates[[step_to_index]])
  }
  
  # Main program logic starts here!
  
  # rewrote this 11/21
  if (direction == "forward") {
    
    # build and store the empty model...
    #cat("Building empty model\n")
    empty_model <- GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta) 
    #empty_model <- ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta)
    steps_taken <- store_model(empty_model, steps_taken)
    
    # ...then start stepping
    for (size in 1:ncol(target_model$X)) { 
      # candidates vector contains all models to choose from at this step, index 1 is the current model
      candidates <- list()
      candidates[[1]] <- steps_taken$current_model
      current_covariates <- candidates[[1]]$covariates
      combs <- combinations(current_covariates, size)
      for (i in 1:nrow(combs)) {
        combination <- combs[i,]
        #cat("Building", combination, "\n")
        #candidates[[i+1]] <- ARCHmX(X, p, eps, covariates=combination, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice)
        candidates[[i+1]] <- generate_candidate_model(target_model, covariates = combination) #### ?????????
      }
      
      # attempt a step...
      best_model <- attempt_step(candidates, step_direction="forward")    # ????
      if (identical(steps_taken$current_model, best_model)) {
        break # ...exit the loop if you don't take one
      } else {
        steps_taken <- store_model(best_model, steps_taken)
      }
    }

  } else if (direction == "backward") { 
    
    # build and store the full model...
    full_model <- target_model
    steps_taken <- store_model(full_model, steps_taken)
    
    # ...then start stepping
    for (size in ((ncol(target_model$X)-1):0)) {
      candidates <- list()
      candidates[[1]] <- steps_taken$current_model
      current_covariates <- candidates[[1]]$covariates
      if (size > 0) { # Candidates are covariate models
        combs <- combinations(current_covariates, size)
        for (i in 1:nrow(combs)) {
          combination <- combs[i,]
          #candidates[[i+1]] <- ARCHmX(X, p, eps, covariates=combination, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice)
          candidates[[i+1]] <- generate_candidate_model(target_model, covariates = combination)
        }
      } else { # Candidate is the empty model
        #candidates[[2]] <- ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta)
        candidates[[2]] <- GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta) 
      }

      # attempt a step...
      best_model <- attempt_step(candidates, step_direction="backward")
      if (identical(steps_taken$current_model, best_model)) {
        break # ...exit the loop if you don't take one
      } else {
        steps_taken <- store_model(best_model, steps_taken)
      }
    }
    
    # best subset logic written 11/23
  } else if (direction == "bestsubset") {
    # Retain the model with best metric performance and all significant p-values. 
    # If no model has all significant p-values, return the empty model.
    
    best_metric <- Inf # we want to minimize all metrics)
    best_model <- list()
    all_covariates <- 1:ncol(target_model$X)
    for (size in 1:ncol(target_model$X)) {
      combs <- combinations(all_covariates, size)
      for (i in 1:nrow(combs)) {
        combination <- combs[i,]
        #candidate<- ARCHmX(X, p, eps, covariates=combination, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice)
        candidate <- generate_candidate_model(target_model, covariates = combination)
        candidate_metric <- metric(candidate, criterion)
        candidate_max_adj_pval <- metric(candidate, "pvalue") 
        improves_metric <- (candidate_metric < best_metric)
        adj_pval_signif <- (candidate_max_adj_pval < alpha.level)
        if (improves_metric & adj_pval_signif) {
          best_metric <- candidate_metric
          best_model <- candidate
        }
      }  
    }
    if (best_metric == Inf) { # if no candidate found...
      #steps_taken <- store_model(ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta), steps_taken) # ...return the empty model
      steps_taken <- store_model(GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta), steps_taken) # ...return the empty model
    } else { # otherwise return the best candidate
      steps_taken <- store_model(best_model, steps_taken)
    }

    # both directions case
  } else if (direction %in% c("both_null", "both_full")) {
    # Validation
    if (criterion == "pvalue") {
      #stop("Error: criterion 'pvalue' cannot be used for 'both'.")
      stop("Error: criterion 'pvalue' cannot be used for 'both_null' or 'both_full'.")
    }
    
    # Set flag
    converged <- FALSE
    
    # Build initial model 
    if (direction == "both_null") {
      #initial_model <- ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta)
      initial_model <- GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta)
    } else { # direction == "both_full"
      #initial_model <- ARCHmX(X_initial, p, eps, delta=delta, hypothesisTest=TRUE)
      initial_model <- target_model
    }
    steps_taken <- store_model(initial_model, steps_taken)
    
    while(!converged) {
      # get current model
      candidates <- list()
      candidates[[1]] <- steps_taken$current_model
      current_covariate_count <- covariate_count(candidates[[1]])
      
      # get forward candidates if current model not full model
      if (current_covariate_count < ncol(target_model$X)) {
        current_covariates <- candidates[[1]]$covariates
        forward_combs <- combinations(current_covariates, current_covariate_count + 1)
        for (i in 1:nrow(forward_combs)) {
          combination <- forward_combs[i,]
          #candidates[[1 + i]] <- ARCHmX(X, p, eps, covariates=combination, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice)
          candidates[[1 + i]] <- generate_candidate_model(target_model, covariates = combination)
        }
      }
      
      # we only want backward candidates when 
      #   1. current model is not the empty model
      #   2. current model doesn't have all significant p values
      current_model_empty <- (current_covariate_count == 0)
      if (current_model_empty) {
        all_pvals_signif <- TRUE
      } else {
        all_pvals_signif <- metric(candidates[[1]], "pvalue") <= alpha.level
      }
      if (!(current_model_empty | all_pvals_signif)) {
        n_candidates <- length(candidates)
        if (current_covariate_count >= 2) {
          backward_combs <- combinations(candidates[[1]]$covariates, current_covariate_count - 1)
          for (i in 1:nrow(backward_combs)) {
            combination <- backward_combs[i,]
            #candidates[[n_candidates + i]] <- ARCHmX(X, p, eps, covariates=combination, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice)
            candidates[[n_candidates + i]] <- generate_candidate_model(target_model, covariates = combination)
          }
        } else { #length(current_covariates)==1 so the backwards candidate is the empty model
          #candidates[[n_candidates + 1]] <- ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta)
          candidates[[n_candidates + 1]] <- GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta)
          
        }
      }
      
      best_model <- attempt_step(candidates, step_direction="both")
      
      if (identical(steps_taken$current_model, best_model)) {
        converged <- TRUE
      } else {
        steps_taken <- store_model(best_model, steps_taken)
      }
    }
    # one-pass case, written 12/6
  } else {
    if (criterion != "pvalue") {
      stop("Error: criterion must be 'pvalue' to use 'onepass'.")
    }
    full_model <- target_model
    significant_covariates <- full_model$covariates[full_model$covariate_p.values < alpha.level]
    if (length(significant_covariates) > 0) {
      #return(ARCHmX(X, p, eps, covariates=significant_covariates, delta=delta, hypothesisTest=TRUE, bandwidth = bandwidth, k.choice = k.choice))
      return(generate_candidate_model(target_model, covariates=significant_covariates))
    }
    else {
      #return(ARCH_internal(eps=target_model$eps, p=target_model$p, delta=target_model$delta))
      return(GARCHX(eps = target_model$eps, X = NULL, order = c(target_model$p,0), delta = target_model$delta))
    }
  }
  
  cat("Selection process complete. Steps taken:\n\n")
  steps.info <- data.frame(Covariates = steps_taken$covariates, 
                           AIC = steps_taken$aic, 
                           BIC = steps_taken$bic, 
                           MaxPValue = steps_taken$maxpval)
  n_steps <- nrow(steps.info) - 1
  rownames(steps.info) <- 0:n_steps
  selected_model <- steps_taken$current_model
  selected_model$steps.info <- steps.info
  cat("Selection process complete. Total steps taken:", n_steps, "\n\nLast model returned as selected model.")
  return(selected_model)
}

# approach B: smooth residuals and then shift upward by subtracting the minimum so it’s nonnegative

ARCHmXAdditive_backfitting_lowes <- function(target_sum_nonpar, X, bandwidth = NULL,
                              max_iter = 50, tol = 1e-4, floor_eps = 1e-8) {
  # target_sum_nonpar: vector of length n with the "total" nonparametric component
  #                    i.e., sigma2_resid = eps^2 - arch_component   (can be < 0)
  # We fit m_ell >= 0 via m_ell(x) = g_ell(x) - min(g_ell(x)).
  
  n <- nrow(X) # number of samples
  d <- ncol(X) # number of covariates
  
  # Initialize m values strictly positive (tiny), and storage for functions
  m_values <- matrix(floor_eps, nrow = n, ncol = d)
  m_funcs  <- vector("list", d)
  
  # bandwidths
  if (is.null(bandwidth)) {
    bandwidth <- rep(NA_real_, d)
    for (ell in 1:d) {
      x_range <- diff(range(X[, ell]))
      bandwidth[ell] <- 0.5 * x_range * n^(-1/5)
    }
  }
  
  for (iter in 1:max_iter) {
    m_old <- m_values
    
    for (ell in 1:d) {
      # Partial residual for component ell
      partial_resid <- target_sum_nonpar - rowSums(m_values[, -ell, drop = FALSE])
      
      fit <- lowess(X[, ell], partial_resid, f=.3)
      fit$y = fit$y - min(fit$y) # shift curve to ensure nonnegative
      
      # Predict on observed X and exponentiate back
      ghat_at_x  <- approx(fit$x, fit$y, xout = X[, ell], rule = 2)$y
      m_values[, ell] <- ghat_at_x
      
      # Store the function on a grid
      m_funcs[[ell]] <- (function(xgrid, ygrid) {
        # returns m_ell(x) = exp(g_ell(x))
        f_lin <- approxfun(xgrid, ygrid, rule = 2)
        function(z) f_lin(z)
      })(fit$x, fit$y)
    }
    
    if (max(abs(m_values - m_old)) < tol) break
  }
  
  list(m_values = m_values, m_funcs = m_funcs, bandwidth = bandwidth)
}

# --- Nadaraya–Watson predictions at Xnew using training (Xtrain, ytrain) -
ARCHmX_predict_nw_iso <- function(Xtrain, ytrain, Xnew, h) {
  K <- ARCHmX_kernel_between_iso(Xnew, Xtrain, h)     # n_new x n_train
  num <- as.vector(K %*% ytrain)                      # row sums of K * y
  den <- pmax(rowSums(K), .Machine$double.eps)        # avoid divide-by-zero
  num / den
}

# approach A: truncate residuals to floor_eps, smooth log, then exponentiate to ensure nonnegative

ARCHmXAdditive_backfitting <- function(target_sum_nonpar, X, bandwidth = NULL,
                        max_iter = 50, tol = 1e-4, floor_eps = 1e-8) {
  # target_sum_nonpar: vector of length n with the "total" nonparametric component
  #                    i.e., sigma2_resid = eps^2 - arch_component   (can be < 0)
  # We fit m_ell >= 0 via m_ell(x) = exp(g_ell(x)), smoothing log of partial residuals.
  
  n <- nrow(X) # number of samples
  d <- ncol(X) # number of covariates
  
  # Initialize m values strictly positive (tiny), and storage for functions
  m_values <- matrix(floor_eps, nrow = n, ncol = d)
  m_funcs  <- vector("list", d)
  
  # bandwidths
  if (is.null(bandwidth)) {
    bandwidth <- rep(NA_real_, d)
    for (ell in 1:d) {
      x_range <- diff(range(X[, ell]))
      bandwidth[ell] <- 0.5 * x_range * n^(-1/5)
    }
  }
  
  # backfitting
  for (iter in 1:max_iter) {
    m_old <- m_values
    
    for (ell in 1:d) {
      # Partial residual for component ell
      partial_resid <- target_sum_nonpar - rowSums(m_values[, -ell, drop = FALSE])
      
      # Enforce positivity before log
      y_pos <- pmax(partial_resid, floor_eps)
      # Smooth log(y_pos) vs X[,ell]
      fit <- locpoly(X[, ell], log(y_pos), bandwidth = bandwidth[ell],
                     degree = 1, gridsize = max(100, n), range.x = range(X[, ell])) # from KernSmooth
      
      # if (any(!is.finite(fit$y))) {
      #   print("non-finite fit$y:")
      #   print(fit$y[!is.finite(fit$y)])
      # }
      good <- is.finite(fit$x) & is.finite(fit$y)
      
      # Predict on observed X and exponentiate back
      # ghat_at_x  <- approx(fit$x, fit$y, xout = X[, ell], rule = 2)$y
      ghat_at_x <- approx(fit$x[good], fit$y[good], xout = X[, ell], rule = 2)$y
      ghat_at_x[!is.finite(ghat_at_x)] <- log(floor_eps) # log because we exponentiate later
      cap <- log(.Machine$double.xmax) - 1
      ghat_at_x <- pmin(ghat_at_x, cap)

      # if (any(!is.finite(ghat_at_x))) {
      #   print("non-finite ghat_at_x:")
      #   print(ghat_at_x[!is.finite(ghat_at_x)])
      # }
      
      # m_values[, ell] <- pmax(replace(exp(ghat_at_x), !is.finite(exp(ghat_at_x)), floor_eps), floor_eps)
      m_values[, ell] <- pmax(exp(ghat_at_x), floor_eps)
  
      # Store the function on a grid
      m_funcs[[ell]] <- (function(xgrid, ygrid) {
        # returns m_ell(x) = exp(g_ell(x))
        f_lin <- approxfun(xgrid, ygrid, rule = 2)
        function(z){
          val <- f_lin(z)
          val[!is.finite(val)] <- log(floor_eps)
          val <- pmin(val, cap)
          pmax(exp(val), floor_eps)
          # pmax(exp(f_lin(z)), floor_eps) # 
        } # return m_hat (max(g_hat, floor_eps))
      })(fit$x, fit$y)
    }
    
    # if (max(abs(m_values - m_old)) < tol) break
    change <- m_values - m_old
    change[!is.finite(change)] <- Inf
    if (max(abs(change)) < tol) break
  }
  
  list(m_values = m_values, m_funcs = m_funcs, bandwidth = bandwidth)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARCHmX.R
\name{ARCHmX}
\alias{ARCHmX}
\title{Fitting an ARCH-m(X) model}
\usage{
ARCHmX(
  eps, 
  X, 
  p, 
  delta=2,
  covariates="all",
  hypothesisTest=TRUE,
  bandwidth = "plugin", 
  k.choice = "plugin"
)
}
\arguments{
\item{eps}{Time series}

\item{X}{Matrix with exogenous covariates where the number of rows is equal to the length of eps and columns represent the covariates.}

\item{p}{Order of the ARCH-m(X) model. Value of p cannot be 0.}

\item{delta}{Value of the power of the main time series for a POWER-GARCH, default is 2.}

\item{covariates}{Instead of slicing X when building partial models, you can specify a vector of column indices. Defaults to "all".}

\item{hypothesisTest}{Whether to perform covariate hypothesis testing, default is TRUE.}

\item{bandwidth}{Choice of bandwidth for the Nadaraya-Watson Kernel regression estimation. Options are "plugin" or "CV". Default is "plugin".}

\item{k.choice}{Choice of window size k for the ANOVA hypotghesis test. Options are "plugin" or "CV". Default is "plugin".}
}
\value{
An object of class ARCHmX
}
\description{
Fits an ARCH-m(X) model with given data and estimates the alphas and the nonparametric function m(X). 
}
\details{
Performs semiparametric estimation of \eqn{\alpha_1,\cdots,\alpha_p} and \eqn{m(X_{t})} for the ARCH-m(X) model \deqn{\epsilon_t=z_t\sigma_t,}
\deqn{\sigma_t^2=\sum_{j=1}^p\alpha_j\epsilon_{t-j}^2+m(\mathbf{X}_{t-1}).}

Under the model, it can be shown that 
\deqn{\mathbb{E}\left[\epsilon_t^2-\sum_{j=1}^p\alpha_j\epsilon_{t-j}^2\right]=m(\mathbf{X}_{t-1}).}

A Nadaraya-Watson kernel is used to estimate \eqn{m(\mathbf{x}_{t-1})} as
\deqn{\hat{m}(\mathbf{x}_{t-1})=\hat{m}(\mathbf{x}_{t-1},\mathbf{\alpha})=\sum_{\ell=p+1}^{T}W_{\ell}(\mathbf{x}_{t-1})\left(\epsilon_\ell^2-\sum_{j=1}^p\alpha_j\epsilon_{\ell-j}^2\right).}

This gives rise to a residual sum of squares
\deqn{\text{RSS}=\sum_{t=p+1}^T\left(\tilde{\epsilon}_{t,0}^2-\sum_{k=1}^p\alpha_k\tilde{\epsilon}_{t,k}^2\right)^2=\|\tilde{\mathbf{Y}}-\tilde{\mathbf{X}}\alpha\|^2}

where
\deqn{\tilde{\epsilon}_{t,k}^2=\epsilon_{t-k}^2-\sum_{\ell=p+1}^TW_\ell(\mathbf{x}_{t-1})\epsilon_{\ell-k}^2.}

Non-negative least squares regression is used to obtain the final estimates of \eqn{\alpha_1,\cdots,\alpha_p}.

No variable selection is done in this function, but an ANOVA-type hypothesis test may be performed for the significance of each covariate. This is enabled by default. 
}
\examples{
# We simulate a time series as in the simulate_ARCHmX example.
set.seed(1)
n <- 2000
alpha <- c(0.01, 0.02, 0.03)
valinit <- 200
X <- matrix(rnorm((n + valinit) * 5), ncol = 5)

m <- function(X) {
  abs(X[, 1])
}

data <- sim.ARCHmX(n = n, alpha = alpha, m = m, X = X, delta = 2, valinit = valinit)

p <- length(alpha)

# Then we fit the model
model <- ARCHmX(eps = data$eps, X = data$X, p=p)

# Model summary shows Covariate 1 significant at the <0.001 level. 
summary(model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_ARCHmX.R
\name{sim.ARCHmX}
\alias{sim.ARCHmX}
\title{Simulating data from an ARCH-m(X) process}
\usage{
sim.ARCHmX(n, 
  alpha, 
  m, 
  X, 
  delta=2, 
  shock.distr = "Normal", 
  valinit=200
)
}
\arguments{
\item{n}{Desired length of simulated time series data}
\item{alpha}{Set of parameters which determines the model's \eqn{p}-step time dependence.}
\item{m}{The function to apply to \eqn{X}. m must accept the matrix \eqn{X} and produce a vector of outputs for every row of \eqn{X}.}
\item{X}{Matrix with exogenous covariates where each row is an observation. Must be compatible with \eqn{m}}
\item{delta}{Value of the power of the main time series for a POWER-GARCH, default is 2}
\item{shock.distr}{The distribution from which the epsilons will be simulated. Default is 'Normal', and must be one of 'Normal', 't5', 't7', or 'Laplace'.}
\item{valinit}{How many cycles the simulation should spend warming up before returning samples. Default is 200}
}
\value{
A vector of simulated \eqn{\epsilon}s corresponding to each observation (row) in \eqn{X}.
}
\description{
Returns a sample ARCH-m(X) time series given \eqn{\alpha}s, data \eqn{X}, and a function \eqn{m(X)} provided by the user. A vector of simulated \eqn{\epsilon}s will be returned for every row in \eqn{X}.
}
\details{
The user's specified \eqn{\alpha}s, \eqn{\delta}, function \eqn{m(X)}, data \eqn{X}, and shock distribution \eqn{Z} partially define an ARCH-m(X) time series
\deqn{
\begin{cases}
    \epsilon_t=z_t\sigma_t\\
    \sigma_t^\delta=\sum_{j=1}^p\alpha_j\epsilon_{t-j}^\delta+m(\mathbf{X}_{t-1})
\end{cases}}
To get the \eqn{\epsilon_t} simulations off the ground, a series of burn-in \eqn{\epsilon}s are generated and used to build each \eqn{\sigma_t^\delta}. From here, the shock distribution is sampled to produce each \eqn{\epsilon_t}. 
}
\examples{
set.seed(1)
n <- 2000
alpha <- c(0.01, 0.02, 0.03)
valinit <- 200
X <- matrix(rnorm((n + valinit) * 5), ncol = 5)

m <- function(X) {
  abs(X[, 1])
}

data <- sim.ARCHmX(n = n, alpha = alpha, m = m, X = X, delta = 2, valinit = valinit)

summary(data$eps)
plot(data$eps, type = "l", main = "Simulated ARCH-m(X) series")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.ARCHmXAdditive.R
\name{sim.ARCHmXAdditive}
\alias{sim.ARCHmXAdditive}
\title{Simulate data from an ARCHmXAdditive model}
\usage{
sim.ARCHmXAdditive(n, 
  alpha, 
  m, 
  X, 
  delta = 2, 
  valinit = 200, 
  shock.distr = "Normal"
)
}
\arguments{
\item{n}{Desired length of simulated time series data.}
\item{alpha}{Set of parameters which determines the model's \eqn{p}-step time dependence.}
\item{m}{List of length \code{ncol(X)} containing univariate functions \eqn{m_1,\ldots,m_d}. Each \code{m[[ell]]} must accept a single numeric input and return a single finite nonnegative numeric value. The additive term is \eqn{\sum_{\ell=1}^d m_\ell(X_{t-1,\ell})}.}
\item{X}{Matrix with exogenous covariates where each row is an observation. Must be compatible with \code{m}.}
\item{delta}{Value of the power of the main time series for a POWER-GARCH, default is 2.}
\item{valinit}{Integer burn-in length used to stabilize the recursion before returning the simulated sample. Default is 200.}
\item{shock.distr}{Innovation (shock) distribution for \eqn{z_t}. One of \code{"Normal"}, \code{"t5"}, \code{"t7"}, or \code{"Laplace"}.}
}
\value{
A vector of simulated \eqn{\epsilon}s corresponding to each observation (row) in \eqn{X}.
}
\description{
Returns a sample Additive ARCH-m(X) time series given \eqn{\alpha}s, data \eqn{X}, and list of functions \code{m} provided by the user. A vector of simulated \eqn{\epsilon}s will be returned for every row in \eqn{X}.
}
\examples{
set.seed(1)
n <- 2000
alpha <- c(0.2, 0.1)
valinit <- 200
X <- matrix(rnorm((n+valinit) * 3), ncol = 3)

m <- list(
  function(x) 4 * abs(sin(x)),
  function(x) 2 * x^2,
  function(x) 1.5
)

data <- sim.ARCHmXAdditive(n = n, alpha = alpha, X = X, m = m)
summary(data$eps)
plot(data$eps, type = "l", main = "Simulated additive ARCH-m(X) series")
}

\name{ARCHmXAdditive}
\alias{ARCHmXAdditive}
\title{Creating and fitting an Additive ARCH-m(X) model for variable selection}
\usage{
ARCHmXAdditive(
  eps, 
  X, 
  p, 
  delta=2,
  covariates="all",
  method="smooth_means_Positive_Constraint_Exp",
  train_end = NULL, 
  n_burn = 10, 
  max_outer_iter = 30, 
  tol = 1e-4,
  hypothesisTest=TRUE,
  k.choice = "plugin"
)
}
\arguments{
\item{eps}{Time series.}
\item{X}{Matrix with exogenous covariates where the number of rows is equal to the length of eps and columns represent the covariates}
\item{p}{Order of the Additive ARCH-m(X) model. Value of p cannot be 0.}
\item{delta}{Value of the power of the main time series for a POWER-GARCH. Default is 2.}
\item{covariates}{Instead of slicing X when building partial models, you can specify a vector of column indices. Defaults to "all".}
\item{method}{Backfitting strategy used for the additive nonparametric component. One of \code{"smooth_means_Positive_Constraint_Exp"}, \code{"smooth_means_lowes"}, or \code{"smooth_variance"}.
  \tabular{ll}{
    \code{"smooth_means_Positive_Constraint_Exp"} \tab Smooth log-partial-residuals and exponentiate (log/exp positivity constraint).\cr
    \code{"smooth_means_lowes"} \tab Smooth partial residuals and shift upward by subtracting the minimum (smooth + shift).\cr
    \code{"smooth_variance"} \tab Smooth a local weighted variance curve of partial residuals, take a square root, then shift upward (variance-smooth + shift).\cr
  }
}
\item{train_end}{Optional integer. If provided, fit on \code{1:train_end} and compute test metrics on the remainder. If not provided, fit on the full sample.}
\item{n_burn}{Integer burn-in. Iterations and fitted quantities are evaluated only after burn-in has passed. Default is 10.}
\item{max_outer_iter}{Maximum number of outer backfitting iterations. Default is 30.}
\item{tol}{Convergence tolerance for the outer iterations. Default is \code{1e-4}.}
\item{hypothesisTest}{Whether to perform covariate hypothesis testing, default is TRUE.}
\item{k.choice}{Choice of window size k for the ANOVA hypotghesis test. Options are "plugin" or "CV". Default is "plugin".}
}

\value{
An object of class ARCHmXAdditive
}

\description{
Fits an Additive ARCH-m(X) model with given data and estimates the alphas and nonparametric functions m(X). 
}

\details{
This function fits an additive semiparametric volatility model of the form
\deqn{\epsilon_t = \sigma_t w_t,}
\deqn{\sigma_t^2 = \sum_{i=1}^{p}\alpha_i |\epsilon_{t-i}|^2 + \sum_{\ell=1}^{d} m_{\ell}(X_{t-1,\ell}).}

Estimation proceeds by iterating between:
\enumerate{
\item Updating the ARCH parameters \eqn{\alpha_1,\ldots,\alpha_p} (subject to nonnegativity constraints), and
\item Updating the additive components \eqn{m_{\ell}} using backfitting with a positivity-enforcing strategy controlled by \code{method}.
}

The outer loop stops when the change in the objective (or updates) is below \code{tol} or when \code{max_outer_iter} is reached.

No variable selection is done in this function, but an ANOVA-type hypothesis test may be performed for the significance of each covariate. This is enabled by default. 
}

\examples{
\donttest{
# We simulate a time series as in the simulate.ARCHmXAdditive example.
set.seed(1)
n <- 2000
alpha <- c(0.2, 0.1)
valinit <- 200
X <- matrix(rnorm((n+valinit) * 3), ncol = 3)

m <- list(
  function(x) 4 * abs(sin(x)),
  function(x) 2 * x^2,
  function(x) 1.5
)

data <- sim.ARCHmXAdditive(n = n, alpha = alpha, X = X, m = m)
p <- length(alpha)

# Then we fit the model
model <- ARCHmXAdditive(X = data$X, p = p, eps = data$eps, 
                        method = "smooth_means_lowes", max_outer_iter = 15)
model$alphas
model$converged

# We inspect the model summary
summary(model)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise_ARCHmX.R
\name{stepwise}
\alias{stepwise}
\title{Performing Stepwise Variable Selection for an ARCH-m(X) or GARCHX Process}
\usage{
stepwise(
  target_model, 
  direction="forward",
  criterion="pvalue",
  adjust_method="fdr",
  alpha.level=0.05,
  trace = TRUE
)
}
\arguments{
\item{target_model}{A model object of class \code{ARCHmX}, \code{ARCHmXAdditive}, or \code{GARCHX}. This model must be fit on all available covariates and, in the case of \code{ARCHmX} and \code{ARCHmXAdditive}, the covariate hypothesis testing must have been set to \code{TRUE}. This model defines all the relevant attributes of the candidate models the stepwise selection process will consider.}
\item{direction}{The mode of stepwise search, can be one of "forward", "backward", "both_null" (both directions starting with the empty model), "both_full" (both directions starting with the full model), "bestsubset", and "onepass", with a default of "forward".}
\item{criterion}{The criterion for taking additional steps, default is "pvalue".}
\item{adjust_method}{Correction for multiple testing accumulation of error. See \link{p.adjust}.}
\item{alpha.level}{Defines the adjusted p-value significance level, default is 0.05.}
\item{trace}{If `TRUE`, information is printed for each step of variable selection. Default is `FALSE`. Offers summaries of prospective models as each predictor in the scope is added to or removed from the model. `max_pvalue` indicates the maximum p-value from the multiple tests for each predictor in the model.}
}
\value{
The final model chosen by the stepwise-selection process. 
}
\description{
Returns the fitted ARCH-m(X), additive ARCH-m(X), or GARCHX model resulting from the specified stepwise selection algorithm. 
}
\details{
The \code{forward} selection routine starts with a (null or empty) ARCH model. At each iteration, the current model is compared to an array of all possible models with one additional covariate. A forward step is taken to the candidate model with all significant covariates providing the most improvement in the specified criterion. If no such model exists, or if all covariates are exhausted, the current model is returned. 

The \code{backward} selection routine starts with the full ARCH-m(X), additive ARCH-m(X), or GARCHX model with all possible covariates. At each iteration, the current model is compared to an array of all possible models with one covariate removed. A backward step is taken to the candidate model providing the most improvement in the specified criterion until a model is found with all significant covariates, or until the logic reaches the empty model.

The \code{both_null} and \code{both_full} selection routines start with the null and full models, respectively. At each iteration, both forward and backward steps are attempted as outlined above. This continues until no step is taken, and the current model will be returned. The \code{pvalue} criterion can not be used for these routines since the p-value contstraints for forward and backward selection are incompatible with one another. 

The \code{best_subset} selection routine will build models for every possible combination of covariates and return the one with the best value for the specified criterion. 

The \code{onepass} selection routine builds the full model and returns a new model containing every covariate identified as significant within the full model. The only criterion available for \code{onepass} is \code{pvalue}. 

The user may specify p-value corrections to apply to the p-values used within the desired stepwise selection routine. All options provided by \code{p.adjust} are exposed to the user; the default is set to \code{fdr}.
}
\examples{
\donttest{
# We simulate a time series as in the simulate_ARCHmX example.
set.seed(1)
n <- 2000
alpha <- c(0.01, 0.02, 0.03)
valinit <- 200
X <- matrix(rnorm((n + valinit) * 5), ncol = 5)

m <- function(X) {
  abs(X[, 1])
}

data <- sim.ARCHmX(n = n, alpha = alpha, m = m, X = X, delta = 2, valinit = valinit)
p <- length(alpha)

# Output will show the trace from the empty model to the model with Covariate 1, 
# which is returned as stepwise_model.
target_ARCHmX_model <- ARCHmX(X = data$X, p = p, eps = data$eps)
stepwise_model <- stepwise(target_ARCHmX_model, direction="forward", criterion="pvalue") 
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{sim.GARCHX}
\alias{sim.GARCHX}
\title{Simulate GARCHX model}
\usage{
sim.GARCHX(n, 
  omega, 
  alpha, 
  beta, 
  gamma, 
  X, 
  delta = 2, 
  shock.distr = "Normal", 
  valinit = 200
)
}
\arguments{
\item{n}{Desired length of simulated time series data.}
\item{omega}{Coefficient value for omega, required to be \deqn{\omega_0 > 0}}
\item{alpha}{ARCH Coefficient value, required to be \deqn{\alpha_0 \geq 0}}
\item{beta}{GARCH Coefficient value, required to be \deqn{\beta_0 \geq 0}}
\item{gamma}{Vector containing coefficients for exogenous covariates.}
\item{X}{Matrix with exogenous covariates where the number of rows is equal to the length of n + valinit}
\item{delta}{Value of the power of the time series to allow for Power GARCHX, default is 2 for GARCHX}
\item{shock.distr}{Distribution of the shock eta_t that multiply w_t in the GARCH-X model eps_ = w_t*eta_t.}
\item{valinit}{Initialization value, default value is 200}
}
\value{
A named list containing vector of Time Series data and X covariates used
}
\description{
Simulates Time series data from GARCH model with exogenous covariates
}
\examples{
set.seed(1)
n <- 200
d <- 4
valinit <- 100
n2 <- n + d + 1
omega <- 0.05
alpha <- 0.05
beta <- 0.05
delta <- 2
gamma <- rep(0.05, d)
e<-rnorm(n2+valinit)
Y<-e
for (t in 2:n2)
 Y[t]<- 0.2*Y[t-1]+e[t]
x<-exp(Y)
X <- matrix(0, nrow = (n+valinit), ncol = length(gamma))
for(j in 1:d)
 X[, j] <- x[(d+2-j):(n+d+1-j+valinit)]
data <- sim.GARCHX(n = n, omega = omega, alpha = alpha, beta = beta, 
                   delta = delta, X = X, gamma = gamma, valinit = valinit)
}

\name{GARCHX}
\alias{GARCHX}
\title{Fitting a GARCH-X model}
\usage{
GARCHX(
  eps,
  X,
  order = c(1, 1),
  delta = 2,
  covariates = "all",
  optim.method = "NR"
)
}
\arguments{
\item{eps}{Time series}

\item{X}{Matrix with exogenous covariates where the number of rows is equal to the length of eps and columns represent the covariates.}

\item{order}{Order of the GARCH model. Value of p cannot be 0.}

\item{delta}{Value of the power of the main time series to allow for Power GARCHX, default is 2 for GARCHX.}

\item{covariates}{Instead of slicing X when building partial models, you can specify a vector of column indices. Defaults to "all".}

\item{optim.method}{Optimization method for maximizing quasi-likelihood function. Options: "NR", "L-BFGS-B", "GA", "PS", "SA". Default value is "NR".}
}
\value{
An object of class GARCHX
}
\description{
Fits a GARCHX model with given data and estimates the coefficients for omega, alpha, beta, and gamma
}
\details{
Uses the GARCHX model \deqn{\mathcal{E}_t = \sigma_tw_t}
\deqn{\sigma^2_t = \omega_0 + \sum^{p}_{i=1}\alpha_i\mathcal{E}_{t-i}^2 + \sum^q_{j=1}\beta_j\sigma^2_{t-j}+\mathbf{\gamma}^T\mathbf{x}_{t-1}}

To estimate the coefficients for \deqn{\omega, \alpha, \beta, \gamma}.
}
\examples{
# We simulate a time-series as in the simulate.GARCHX example. 
set.seed(1)
n <- 200
d <- 4
valinit <- 100
n2 <- n + d + 1
omega <- 0.05
alpha <- 0.05
beta <- 0.05
delta <- 2
gamma <- rep(0.05, d)
e<-rnorm(n2+valinit)
Y<-e
for (t in 2:n2)
 Y[t]<- 0.2*Y[t-1]+e[t]
x<-exp(Y)
X <- matrix(0, nrow = (n+valinit), ncol = length(gamma))
for(j in 1:d)
 X[, j] <- x[(d+2-j):(n+d+1-j+valinit)]
data <- sim.GARCHX(n = n, omega = omega, alpha = alpha, beta = beta, 
                   delta = delta, X = X, gamma = gamma, valinit = valinit)

# Then we fit the model
model <- GARCHX(data$eps, data$X, order = c(1, 1), delta = 2, 
                covariates = "all", optim.method = "NR")

# We inspect the model summary
summary(model)
}
